# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Unittest for the NameChecker."""

import re

import astroid

from pylint.checkers import base
from pylint.interfaces import HIGH
from pylint.testutils import CheckerTestCase, MessageTest, set_config


class TestMultiNamingStyle(CheckerTestCase):
    CHECKER_CLASS = base.NameChecker

    MULTI_STYLE_RE = re.compile("(?:(?P<UP>[A-Z]+)|(?P<down>[a-z]+))$")

    @set_config(class_rgx=MULTI_STYLE_RE)
    def test_multi_name_detection_majority(self) -> None:
        classes = astroid.extract_node(
            """
        class classb(object): #@
            pass
        class CLASSA(object): #@
            pass
        class CLASSC(object): #@
            pass
        """
        )
        message = MessageTest(
            "invalid-name",
            node=classes[0],
            args=(
                "Class",
                "classb",
                "the `UP` group in the '(?:(?P<UP>[A-Z]+)|(?P<down>[a-z]+))$' pattern",
            ),
            confidence=HIGH,
            line=2,
            col_offset=0,
            end_line=2,
            end_col_offset=12,
        )
        with self.assertAddsMessages(message):
            cls = None
            for cls in classes:
                self.checker.visit_classdef(cls)
            if cls:
                self.checker.leave_module(cls.root)

    @set_config(class_rgx=MULTI_STYLE_RE)
    def test_multi_name_detection_first_invalid(self) -> None:
        classes = astroid.extract_node(
            """
        class class_a(object): #@
            pass
        class classb(object): #@
            pass
        class CLASSC(object): #@
            pass
        """
        )
        messages = [
            MessageTest(
                "invalid-name",
                node=classes[0],
                args=(
                    "Class",
                    "class_a",
                    "'(?:(?P<UP>[A-Z]+)|(?P<down>[a-z]+))$' pattern",
                ),
                confidence=HIGH,
                line=2,
                col_offset=0,
                end_line=2,
                end_col_offset=13,
            ),
            MessageTest(
                "invalid-name",
                node=classes[2],
                args=(
                    "Class",
                    "CLASSC",
                    "the `down` group in the '(?:(?P<UP>[A-Z]+)|(?P<down>[a-z]+))$' pattern",
                ),
                confidence=HIGH,
                line=6,
                col_offset=0,
                end_line=6,
                end_col_offset=12,
            ),
        ]
        with self.assertAddsMessages(*messages):
            cls = None
            for cls in classes:
                self.checker.visit_classdef(cls)
            if cls:
                self.checker.leave_module(cls.root)

    @set_config(
        method_rgx=MULTI_STYLE_RE,
        function_rgx=MULTI_STYLE_RE,
        name_group=("function:method",),
    )
    def test_multi_name_detection_group(self) -> None:
        function_defs = astroid.extract_node(
            """
        class First(object):
            def func(self): #@
                pass

        def FUNC(): #@
            pass
        """,
            module_name="test",
        )
        message = MessageTest(
            "invalid-name",
            node=function_defs[1],
            args=(
                "Function",
                "FUNC",
                "the `down` group in the '(?:(?P<UP>[A-Z]+)|(?P<down>[a-z]+))$' pattern",
            ),
            confidence=HIGH,
            line=6,
            col_offset=0,
            end_line=6,
            end_col_offset=8,
        )
        with self.assertAddsMessages(message):
            func = None
            for func in function_defs:
                self.checker.visit_functiondef(func)
            if func:
                self.checker.leave_module(func.root)

    @set_config(
        function_rgx=re.compile("(?:(?P<ignore>FOO)|(?P<UP>[A-Z]+)|(?P<down>[a-z]+))$")
    )
    def test_multi_name_detection_exempt(self) -> None:
        function_defs = astroid.extract_node(
            """
        def FOO(): #@
            pass
        def lower(): #@
            pass
        def FOO(): #@
            pass
        def UPPER(): #@
            pass
        """
        )
        message = MessageTest(
            "invalid-name",
            node=function_defs[3],
            args=(
                "Function",
                "UPPER",
                "the `down` group in the '(?:(?P<ignore>FOO)|(?P<UP>[A-Z]+)|(?P<down>[a-z]+))$' pattern",
            ),
            confidence=HIGH,
            line=8,
            col_offset=0,
            end_line=8,
            end_col_offset=9,
        )
        with self.assertAddsMessages(message):
            func = None
            for func in function_defs:
                self.checker.visit_functiondef(func)
            if func:
                self.checker.leave_module(func.root)



shop = {
    # animal: (specie, descriptions)
    "parrot": ("Norvegian blue", ("restin'", "remarkable", "beautiful plumage")),
}

if "parrot" in shop and "restin'" in shop["parrot"][1]:
    print("Hellooooo, Pooolllllyyy ! WAAAAKEEY, WAKKEEEY !")



# pylint: disable = invalid-name, line-too-long
"""Simple test sets for checking duplicate values"""

set1 = {1, 2, 3, 4}
set2 = {1, 1, 2} # [duplicate-value]
set3 = {1, 2, 2} # [duplicate-value]

set4 = {'one', 'two', 'three'}
set5 = {'one', 'two', 'one'} # [duplicate-value]
set6 = {'one', 'two', 'two'} # [duplicate-value]

wrong_set = {12, 23, True, 6, True, 0, 12} # [duplicate-value, duplicate-value]
correct_set = {12, 13, 23, 24, 89}

wrong_set_mixed = {1, 2, 'value', 1} # [duplicate-value]
wrong_set = {'arg1', 'arg2', False, 'arg1', True} # [duplicate-value]

another_wrong_set = {2, 3, 'arg1', True, 'arg1', False, True} # [duplicate-value, duplicate-value]



class Animal:
    beaver_tailed: bool
    can_swim: bool
    has_beak: bool
    has_fur: bool
    has_vertebrae: bool
    lays_egg: bool
    protected_specie: bool
    venomous: bool


class Invertebrate(Animal):
    has_vertebrae = False


class Vertebrate(Animal):
    has_vertebrae = True


class Mammal(Vertebrate):
    has_beak = False
    has_fur = True
    lays_egg = False
    venomous = False


class Playtypus(Mammal):
    beaver_tailed = True
    can_swim = True
    has_beak = True
    lays_egg = False
    protected_specie = True
    venomous = True



"""Example cases for dict-init-mutate"""
# pylint: disable=use-dict-literal, invalid-name

base = {}

fruits = {}
for fruit in ["apple", "orange"]:
    fruits[fruit] = 1
    fruits[fruit] += 1

count = 10
fruits = {"apple": 1}
fruits["apple"] += count

config = {}  # [dict-init-mutate]
config['pwd'] = 'hello'

config = {}  # [dict-init-mutate]
config['dir'] = 'bin'
config['user'] = 'me'
config['workers'] = 5
print(config)

config = {}  # Not flagging calls to update for now
config.update({"dir": "bin"})

config = {}  # [dict-init-mutate]
config['options'] = {}  # Identifying nested assignment not supporting this yet.
config['options']['debug'] = False
config['options']['verbose'] = True


config = {}
def update_dict(di):
    """Update a dictionary"""
    di["one"] = 1

update_dict(config)

config = {}
globals()["config"]["dir"] = "bin"



# pylint: disable=missing-docstring, expression-not-assigned, too-few-public-methods, bad-chained-comparison
# pylint: disable=no-member, import-error, line-too-long
# pylint: disable=unnecessary-comprehension, use-dict-literal, use-implicit-booleaness-not-comparison

from unknown import Unknown


class CustomClass:
    def keys(self):
        return []

for key in Unknown().keys():
    pass
for key in Unknown.keys():
    pass
for key in dict.keys():
    pass
for key in {}.values():
    pass
for key in {}.key():
    pass
for key in CustomClass().keys():
    pass

[key for key in {}.keys()] # [consider-iterating-dictionary]
(key for key in {}.keys()) # [consider-iterating-dictionary]
{key for key in {}.keys()} # [consider-iterating-dictionary]
{key: key for key in {}.keys()} # [consider-iterating-dictionary]
COMP1 = [key for key in {}.keys()] # [consider-iterating-dictionary]
COMP2 = (key for key in {}.keys()) # [consider-iterating-dictionary]
COMP3 = {key for key in {}.keys()} # [consider-iterating-dictionary]
COMP4 = {key: key for key in {}.keys()} # [consider-iterating-dictionary]
for key in {}.keys(): # [consider-iterating-dictionary]
    pass

# Issue #1247
DICT = {'a': 1, 'b': 2}
COMP1 = [k * 2 for k in DICT.keys()] + [k * 3 for k in DICT.keys()]  # [consider-iterating-dictionary,consider-iterating-dictionary]
COMP2, COMP3 = [k * 2 for k in DICT.keys()], [k * 3 for k in DICT.keys()]  # [consider-iterating-dictionary,consider-iterating-dictionary]
SOME_TUPLE = ([k * 2 for k in DICT.keys()], [k * 3 for k in DICT.keys()])  # [consider-iterating-dictionary,consider-iterating-dictionary]

# Checks for membership checks
if 1 in dict().keys(): # [consider-iterating-dictionary]
    pass
if 1 in {}.keys(): # [consider-iterating-dictionary]
    pass
if 1 in Unknown().keys():
    pass
if 1 in Unknown.keys():
    pass
if 1 in CustomClass().keys():
    pass
if 1 in dict():
    pass
if 1 in dict().values():
    pass
if (1, 1) in dict().items():
    pass
if [1] == {}.keys():
    pass
if [1] == {}:
    pass
if [1] == dict():
    pass
VAR = 1 in {}.keys() # [consider-iterating-dictionary]
VAR = 1 in {}
VAR = 1 in dict()
VAR = [1, 2] == {}.keys() in {False}

# Additional membership checks
# See: https://github.com/pylint-dev/pylint/issues/5323
metadata = {}
if "a" not in list(metadata.keys()): # [consider-iterating-dictionary]
    print(1)
if "a" not in metadata.keys(): # [consider-iterating-dictionary]
    print(1)
if "a" in list(metadata.keys()): # [consider-iterating-dictionary]
    print(1)
if "a" in metadata.keys(): # [consider-iterating-dictionary]
    print(1)


class AClass:
    def a_function(self):
        class InnerClass:
            def another_function(self):
                def inner_function():
                    another_metadata = {}
                    print("a" not in list(another_metadata.keys())) # [consider-iterating-dictionary]
                    print("a" not in another_metadata.keys()) # [consider-iterating-dictionary]
                    print("a" in list(another_metadata.keys())) # [consider-iterating-dictionary]
                    print("a" in another_metadata.keys()) # [consider-iterating-dictionary]
                return  inner_function()
        return InnerClass().another_function()

a_dict = {"a": 1, "b": 2, "c": 3}
a_set = {"c", "d"}

# Test bitwise operations. These should not raise msg because removing `.keys()`
# either gives error or ends in a different result
print(a_dict.keys() | a_set)

if "a" in a_dict.keys() | a_set:
    pass

if "a" in a_dict.keys() & a_set:
    pass

if 1 in a_dict.keys() ^ [1, 2]:
    pass

if "a" in a_dict.keys() or a_set:  # [consider-iterating-dictionary]
    pass

if "a" in a_dict.keys() and a_set:  # [consider-iterating-dictionary]
    pass



"""Check for nonlocal and used-before-assignment"""
# pylint: disable=missing-docstring, unused-variable, too-few-public-methods


def test_ok():
    """ uses nonlocal """
    cnt = 1
    def wrap():
        nonlocal cnt
        cnt = cnt + 1
    wrap()

def test_fail():
    """ doesn't use nonlocal """
    cnt = 1
    def wrap():
        cnt = cnt + 1 # [used-before-assignment]
    wrap()

def test_fail2():
    """ use nonlocal, but for other variable """
    cnt = 1
    count = 1
    def wrap():
        nonlocal count
        cnt = cnt + 1 # [used-before-assignment]
    wrap()

def test_fail3(arg: test_fail4): # [used-before-assignment]
    """ Depends on `test_fail4`, in argument annotation. """
    return arg
# +1: [used-before-assignment, used-before-assignment]
def test_fail4(*args: test_fail5, **kwargs: undefined):
    """ Depends on `test_fail5` and `undefined` in
    variable and named arguments annotations.
    """
    return args, kwargs

def test_fail5()->undefined1: # [used-before-assignment]
    """ Depends on `undefined1` in function return annotation. """

def undefined():
    """ no op """

def undefined1():
    """ no op """


def nonlocal_in_ifexp():
    """bar"""
    bug2 = True
    def on_click(event):
        """on_click"""
        if event:
            nonlocal bug2
            bug2 = not bug2
    on_click(True)

nonlocal_in_ifexp()


def type_annotation_only_gets_value_via_nonlocal():
    """https://github.com/pylint-dev/pylint/issues/5394"""
    some_num: int
    def inner():
        nonlocal some_num
        some_num = 5
    inner()
    print(some_num)


def type_annotation_only_gets_value_via_nonlocal_nested():
    """Similar, with nesting"""
    some_num: int
    def inner():
        def inner2():
            nonlocal some_num
            some_num = 5
        inner2()
    inner()
    print(some_num)


def type_annotation_never_gets_value_despite_nonlocal():
    """Type annotation lacks a value despite nonlocal declaration"""
    some_num: int
    def inner():
        nonlocal some_num
    inner()
    print(some_num)  # [used-before-assignment]


def inner_function_lacks_access_to_outer_args(args):
    """Check homonym between inner function and outer function names"""
    def inner():
        print(args)  # [used-before-assignment]
        args = []
    inner()
    print(args)


def inner_function_ok(args):
    """Explicitly redefined homonym defined before is OK."""
    def inner():
        args = []
        print(args)
    inner()
    print(args)



"""Module doc string."""
# pylint: disable=locally-disabled


# pylint: disable=too-few-public-methods
class Base:
    """My base class."""


# pylint: enable=too-few-public-methods


class BaseCorrect:  # pylint: disable=too-few-public-methods
    """My base class."""


class BaseInner:
    # pylint: disable=too-few-public-methods
    """My base class."""



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

import os

import pytest

from pylint.reporters.text import TextReporter
from pylint.testutils._run import _Run as Run

PARENT_DIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
REGR_DATA = os.path.join(PARENT_DIR, "regrtest_data")


@pytest.mark.timeout(8)
def test_process_tokens() -> None:
    with pytest.raises(SystemExit) as cm:
        Run(
            [os.path.join(REGR_DATA, "very_long_line.py"), "--disable=C"],
            reporter=TextReporter(),
        )
    assert cm.value.code == 0


@pytest.mark.timeout(60)
def test_issue_5724() -> None:
    """Regression test for parsing of pylint disable pragma's."""
    with pytest.raises(SystemExit) as cm:
        Run(
            [
                os.path.join(REGR_DATA, "issue_5724.py"),
                "--enable=missing-final-newline",
                "--disable=C",
            ],
            reporter=TextReporter(),
        )
    assert cm.value.code == 0



"""Tests for no-member for self-referencing instance attributes
See https://github.com/pylint-dev/pylint/issues/1555
"""
# pylint: disable=too-few-public-methods


class ClassWithMember:
    """Member defined in superclass."""
    def __init__(self):
        self.member = True


class AssignMemberInSameLine:
    """This class attempts to assign and access a member in the same line."""
    def __init__(self):
        self.member = self.member  # [no-member]


class AssignMemberInSameLineAfterTypeAnnotation:
    """This might emit a message like `maybe-no-member` in the future."""
    def __init__(self):
        self.member: bool
        self.member = self.member


class AssignMemberFromSuper1(ClassWithMember):
    """This assignment is valid due to inheritance."""
    def __init__(self):
        self.member = self.member
        super().__init__()


class AssignMemberFromSuper2(ClassWithMember):
    """This assignment is valid due to inheritance."""
    def __init__(self):
        super().__init__()
        self.member = self.member



from operator import attrgetter

from .nodes import roles


class HasCacheKey(object):
    ...


class HasMemoized(object):
    ...


class MemoizedHasCacheKey(HasCacheKey, HasMemoized):
    ...


class ClauseElement(MemoizedHasCacheKey):
    ...


class ReturnsRows(roles.ReturnsRowsRole, ClauseElement):
    ...


class Selectable(ReturnsRows):
    ...


class FromClause(roles.AnonymizedFromClauseRole, Selectable):
    c = property(attrgetter("columns"))



"""Tests for no-member in relation to AugAssign operations."""
# pylint: disable=missing-module-docstring, too-few-public-methods, missing-class-docstring, invalid-name

# Test for: https://github.com/pylint-dev/pylint/issues/4562
class A:
    value: int

obj_a = A()
obj_a.value += 1  # [no-member]


class B:
    value: int

obj_b = B()
obj_b.value = 1 + obj_b.value  # [no-member]


class C:
    value: int


obj_c = C()
obj_c.value += 1  # [no-member]
obj_c.value = 1 + obj_c.value  # [no-member]



from functools import singledispatchmethod


class Board:
    @singledispatchmethod
    def convert_position(cls, position):
        pass

    @singledispatchmethod
    @classmethod
    def _(cls, position: str) -> tuple:
        position_a, position_b = position.split(",")
        return (int(position_a), int(position_b))

    @singledispatchmethod
    @classmethod
    def _(cls, position: tuple) -> str:
        return f"{position[0]},{position[1]}"



class FasterThanTheSpeedOfLightError(ZeroDivisionError):
    def __init__(self):
        super().__init__("You can't go faster than the speed of light !")


def calculate_speed(distance: float, time: float) -> float:
    try:
        return distance / time
    except ZeroDivisionError as e:
        raise FasterThanTheSpeedOfLightError() from e



"""Test deprecated classes from Python 3.3."""
# pylint: disable=unused-import,import-error,no-name-in-module,abstract-class-instantiated

from collections import Iterable  # [deprecated-class]

import collections.Set  # [deprecated-class]

import collections


_ = collections.Awaitable()  # [deprecated-class]



def foo():  # [docstring-first-line-empty]
    """

    Lorem Ipsum is simply dummy text of the printing and typesetting
    industry.

    Lorem Ipsum has been the industry's standard dummy text ever since the
    1500s, when an unknown printer took a galley of type and scrambled it
    to make a type specimen book
    """



# pylint: disable=missing-docstring,too-many-ancestors, broad-except
import collections.abc
from typing import TYPE_CHECKING, Sized  # pylint: disable=deprecated-class

if TYPE_CHECKING:
    BaseClass = Sized
else:
    BaseClass = collections.abc.Sized


class TestBaseException(BaseClass):
    def __len__(self):
        return 0


class TestException(TestBaseException):
    pass


def test():
    try:
        1 / 0
    except TestException:  # [catching-non-exception,try-except-raise]
        raise
    except Exception:
        pass



"""Logging warnings using a logger class."""
# pylint: disable=consider-using-f-string
from __future__ import absolute_import
import logging


LOG = logging.getLogger("domain")
LOG.debug("%s" % "junk")  # [logging-not-lazy]
LOG.log(logging.DEBUG, "%s" % "junk")  # [logging-not-lazy]
LOG2 = LOG.debug
LOG2("%s" % "junk")  # [logging-not-lazy]

logging.getLogger("domain").debug("%s" % "junk")  # [logging-not-lazy]



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from pathlib import Path

from pylint.testutils._run import _Run as Run


def test_fall_back_on_base_config(tmp_path: Path) -> None:
    """Test that we correctly fall back on the base config."""
    # A file under the current dir should fall back to the highest level
    # For pylint this is ./pylintrc
    test_file = tmp_path / "test.py"
    runner = Run([__name__], exit=False)
    assert id(runner.linter.config) == id(runner.linter._base_config)

    # When the file is a directory that does not have any of its parents in
    # linter._directory_namespaces it should default to the base config
    with open(test_file, "w", encoding="utf-8") as f:
        f.write("1")
    Run([str(test_file)], exit=False)
    assert id(runner.linter.config) == id(runner.linter._base_config)



"""https://github.com/pylint-dev/pylint/issues/3675"""


def noop(x):  # pylint: disable=invalid-name
    """Return value unchanged"""
    return x


def add(x, y):  # pylint: disable=invalid-name
    """Add two values"""
    return x + y


def main(param):
    """Should not emit too-many-function-args"""
    tmp = noop  # matched first
    if param == 0:
        tmp = add
    return tmp(1, 1.01)



"""Tests for used-before-assignment for typing related issues"""
# pylint: disable=missing-function-docstring,ungrouped-imports,invalid-name


from typing import List, Optional, TYPE_CHECKING

if TYPE_CHECKING:
    if True:  # pylint: disable=using-constant-test
        import math
    import dbm
    print(dbm)  # no error when defined and used in the same false branch
    from urllib.request import urlopen
    import array
    import base64
    import binascii
    import calendar
    import collections
    import copy
    import datetime
    import email
    import heapq
    import json
    import mailbox
    import mimetypes
    import numbers
    import pprint
    import types
    import zoneinfo
elif input():
    import calendar, bisect  # pylint: disable=multiple-imports
    if input() + 1:
        import heapq
    else:
        import heapq
elif input():
    try:
        numbers = None if input() else 1
        import array
    except Exception as e:  # pylint: disable=broad-exception-caught
        import types
    finally:
        copy = None
elif input():
    for i in range(1,2):
        email = None
    else:  # pylint: disable=useless-else-on-loop
        json = None
    while input():
        import mailbox
    else:  # pylint: disable=useless-else-on-loop
        mimetypes = None
elif input():
    with input() as base64:
        pass
    with input() as temp:
        import binascii
else:
    from urllib.request import urlopen
    zoneinfo: str = ''
    def pprint():
        pass
    class collections:  # pylint: disable=too-few-public-methods,missing-class-docstring
        pass

class MyClass:
    """Type annotation or default values for first level methods can't refer to their own class"""

    def incorrect_typing_method(
        self, other: MyClass  # [undefined-variable]
    ) -> bool:
        return self == other

    def incorrect_nested_typing_method(
        self, other: List[MyClass]  # [undefined-variable]
    ) -> bool:
        return self == other[0]

    def incorrect_default_method(
        self, other=MyClass()  # [undefined-variable]
    ) -> bool:
        return self == other

    def correct_string_typing_method(self, other: "MyClass") -> bool:
        return self == other

    def correct_inner_typing_method(self) -> bool:
        def inner_method(self, other: MyClass) -> bool:
            return self == other

        return inner_method(self, MyClass())


class MySecondClass:
    """Class to test self referential variable typing.
    This regressed, reported in: https://github.com/pylint-dev/pylint/issues/5342
    """

    def self_referential_optional_within_method(self) -> None:
        variable: Optional[MySecondClass] = self
        print(variable)

    def correct_inner_typing_method(self) -> bool:
        def inner_method(self, other: MySecondClass) -> bool:
            return self == other

        return inner_method(self, MySecondClass())


class MyOtherClass:
    """Class to test self referential variable typing, no regression."""

    def correct_inner_typing_method(self) -> bool:
        def inner_method(self, other: MyOtherClass) -> bool:
            return self == other

        return inner_method(self, MyOtherClass())

    def self_referential_optional_within_method(self) -> None:
        variable: Optional[MyOtherClass] = self
        print(variable)


class MyThirdClass:
    """Class to test self referential variable typing within conditionals.
    This regressed, reported in: https://github.com/pylint-dev/pylint/issues/5499
    """

    def function(self, var: int) -> None:
        if var < 0.5:
            _x: MyThirdClass = self

    def other_function(self) -> None:
        _x: MyThirdClass = self


class MyFourthClass:  # pylint: disable=too-few-public-methods
    """Class to test conditional imports guarded by TYPE_CHECKING two levels
    up then used in function annotation. See https://github.com/pylint-dev/pylint/issues/7539"""

    def is_close(self, comparator: math.isclose, first, second):  # [used-before-assignment]
        """Conditional imports guarded are only valid for variable annotations."""
        comparator(first, second)


class VariableAnnotationsGuardedByTypeChecking:  # pylint: disable=too-few-public-methods
    """Class to test conditional imports guarded by TYPE_CHECKING then used in
    local (function) variable annotations, which are not evaluated at runtime.

    See: https://github.com/pylint-dev/pylint/issues/7609
    and https://github.com/pylint-dev/pylint/issues/7882
    """

    still_an_error: datetime.date  # [used-before-assignment]

    def print_date(self, date) -> None:
        date: datetime.date = date
        print(date)

        import datetime  # pylint: disable=import-outside-toplevel


class ConditionalImportGuardedWhenUsed:  # pylint: disable=too-few-public-methods
    """Conditional imports also guarded by TYPE_CHECKING when used."""
    if TYPE_CHECKING:
        print(urlopen)


class TypeCheckingMultiBranch:  # pylint: disable=too-few-public-methods,unused-variable
    """Test for defines in TYPE_CHECKING if/elif/else branching"""
    def defined_in_elif_branch(self) -> calendar.Calendar:  # [possibly-used-before-assignment]
        print(bisect)  # [possibly-used-before-assignment]
        return calendar.Calendar()

    def defined_in_else_branch(self) -> urlopen:
        print(zoneinfo)  # [used-before-assignment]
        print(pprint())
        print(collections())
        return urlopen

    def defined_in_nested_if_else(self) -> heapq:  # [possibly-used-before-assignment]
        print(heapq)
        return heapq

    def defined_in_try_except(self) -> array:  # [used-before-assignment]
        print(types)  # [used-before-assignment]
        print(copy)  # [used-before-assignment]
        print(numbers)  # [used-before-assignment]
        return array

    def defined_in_loops(self) -> json:  # [used-before-assignment]
        print(email)  # [used-before-assignment]
        print(mailbox)  # [used-before-assignment]
        print(mimetypes)  # [used-before-assignment]
        return json

    def defined_in_with(self) -> base64:  # [used-before-assignment]
        print(binascii)  # [used-before-assignment]
        return base64



"""Tests for logging-not-lazy"""
# pylint: disable=missing-docstring,no-member,deprecated-method,invalid-name, consider-using-f-string

# Muck up the names in an effort to confuse...
import logging as renamed_logging
import os as logging

var = "123"
var_name = "Var:"
# Statements that should be flagged:
renamed_logging.warn("%s, %s" % (4, 5))  # [logging-not-lazy]
renamed_logging.warn("Var: " + var)  # [logging-not-lazy]
renamed_logging.exception("%s" % "Exceptional!")  # [logging-not-lazy]
renamed_logging.log(renamed_logging.INFO, "msg: %s" % "Run!")  # [logging-not-lazy]
renamed_logging.log(renamed_logging.INFO, "Var: " + var)  # [logging-not-lazy]
renamed_logging.log(renamed_logging.INFO, var_name + var)  # [logging-not-lazy]

# Statements that should not be flagged:
renamed_logging.warn("%s, %s", 4, 5)
renamed_logging.log(renamed_logging.INFO, "msg: %s", "Run!")
logging.warn("%s, %s" % (4, 5))
logging.log(logging.INFO, "msg: %s" % "Run!")
logging.log("Var: " + var)
# Explicit string concatenations are fine:
renamed_logging.warn("%s" + " the rest of a single string")
renamed_logging.warn("Msg: " + "%s", "first piece " + "second piece")
renamed_logging.warn("first" + "second" + "third %s", "parameter")
renamed_logging.warn(("first" + "second" + "third %s"))

# Regression crash test for incorrect format call
renamed_logging.error(
    "0} - {1}".format(1, 2)  # [bad-format-string, logging-format-interpolation]
)



"""
https://github.com/pylint-dev/pylint/issues/3507
False-positive 'isinstance-second-argument-not-valid-type'
for typing aliases in 'isinstance' calls.
"""
import collections
import collections.abc
import typing

isinstance(42, typing.Dict)
isinstance(42, typing.Counter)
isinstance(42, typing.Collection)
isinstance(42, typing.Iterator)
isinstance(42, typing.Tuple)
isinstance(42, typing.Callable)
isinstance(42, typing.Type)


# For comparison - also valid calls
isinstance(42, dict)
isinstance(42, collections.Counter)
isinstance(42, collections.abc.Collection)
isinstance(42, collections.abc.Iterator)
isinstance(42, tuple)
isinstance(42, collections.abc.Callable)
isinstance(42, type)



# pylint: disable=missing-docstring,invalid-name
def main1():
    try:
        raise ValueError
    except ValueError as e:  # [unused-variable]
        pass

    print(e)  # [used-before-assignment]


def main2():
    try:
        raise ValueError
    except ValueError as e:
        print(e)


def main3():
    try:
        raise ValueError
    except ValueError as e:  # [unused-variable]
        pass

    e = 10
    print(e)


def main4():
    try:
        raise ValueError
    except ValueError as e:  # [unused-variable]
        pass

    try:
        raise ValueError
    except ValueError as e:
        pass

    try:
        raise ValueError
    except ValueError as e:
        pass

    print(e)  # [used-before-assignment]


def main5():
    try:
        print([e for e in range(3) if e])
    except ValueError as e:
        print(e)



"""Tests for missing-function-docstring and the min-length option"""
# pylint: disable=unused-argument, unnecessary-pass, bare-except


def func(tion):
    pass


def func_two(tion):  # [missing-function-docstring]
    pass
    pass


def func_three(tion):  # [missing-function-docstring]
    try:
        pass
    except:
        pass


def __fun__(tion):
    pass



"""
Here we assume that 'Drink' and 'Cocktail' are different things and should
not be treated together like if they were the same thing.

This will force some downstream changes and force the API user to make a
conscious decision about the alcoholic content of its drink when using the
API. For example, it's impossible to create a mojito with beer without
explicitly wanting to, or to add an alcohol to a soft-drink.
"""


class Drink:
    def mix(self, fluid_one, fluid_two):
        return fluid_one + fluid_two


class Cocktail:
    def mix(self, fluid_one, fluid_two, alcoholic_fluid):
        return fluid_one + fluid_two + alcoholic_fluid



# pylint: disable=missing-docstring,too-few-public-methods,unused-variable,no-name-in-module
from typing import Final

class Foo:
    BAR = "bar"

KEY_3 = "key_3"


# Subdicts have at least 1 common key
MAPPING_1 = {  # [consider-using-namedtuple-or-dataclass]
    "entry_1": {"key_1": 0, "key_2": 1, "key_diff_1": 2},
    "entry_2": {"key_1": 0, "key_2": 1, "key_diff_2": 3},
}
MAPPING_2 = {  # [consider-using-namedtuple-or-dataclass]
    "entry_1": {KEY_3: None, Foo.BAR: None},
    "entry_2": {KEY_3: None, Foo.BAR: None},
}

# ints are not valid fieldnames for namedtuples
MAPPING_3 = {
    "entry_1": {0: None, 1: None},
    "entry_2": {0: None, 1: None},
}

# Subdicts have no common keys
MAPPING_4 = {
    "entry_1": {"key_3": 0, "key_4": 1, "key_diff_1": 2},
    "entry_2": {"key_1": 0, "key_2": 1, "key_diff_2": 3},
}

def func():
    # Not in module scope
    mapping_4 = {
        "entry_1": {"key_1": 0, "key_2": 1},
        "entry_2": {"key_1": 0, "key_2": 1},
    }

    mapping_5: Final = {  # [consider-using-namedtuple-or-dataclass]
        "entry_1": {"key_1": 0, "key_2": 1},
        "entry_2": {"key_1": 0, "key_2": 1},
    }


# lists must have the same length
MAPPING_6 = {  # [consider-using-namedtuple-or-dataclass]
    "entry_1": [1, "a", set()],
    "entry_2": [2, "b", set()],
}
MAPPING_7 = {
    "entry_1": [],
    "entry_2": [],
}
MAPPING_8 = {
    "entry_1": [1],
    "entry_2": [2, "b"],
}
MAPPING_9 = {  # [consider-using-namedtuple-or-dataclass]
    "entry_1": (1, "a"),
    "entry_2": (2, "b"),
}

# No entry can't contain only dicts
MAPPING_10 = {
    "entry_1": [
        {"key_1": None, "key_2": None},
    ],
    "entry_2": [None]
}

# No either dict, tuple, or list as dict values
MAPPING_11 = {
    "entry_1": 1,
    "entry_2": 2,
}
MAPPING_12 = {
    "entry_1": "",
    "entry_2": "",
}



def is_palindrome(string):  # [too-many-lines]
    left_pos = 0
    right_pos = len(string) - 1
    while right_pos >= left_pos:
        if not string[left_pos] == string[right_pos]:
            return False
        left_pos += 1
        right_pos -= 1
    return True


def main():
    print(is_palindrome("aza"))
    print(is_palindrome("racecar"))
    print(is_palindrome("trigger"))
    print(is_palindrome("ogre"))



# pylint: disable=missing-docstring,too-few-public-methods,invalid-name
# pylint: disable=attribute-defined-outside-init


class Aaaa:
    """class with attributes defined in wrong order"""

    def __init__(self):
        var1 = self._var2  # [access-member-before-definition]
        self._var2 = 3
        self._var3 = var1


class Bbbb:
    A = 23
    B = A

    def __getattr__(self, attr):
        try:
            return self.__repo
        except AttributeError:
            self.__repo = attr
            return attr

    def catchme(self, attr):
        """no AttributeError caught"""
        try:
            return self._repo  # [access-member-before-definition]
        except ValueError:
            self._repo = attr
            return attr


class Mixin:
    def test_mixin(self):
        """Don't emit access-member-before-definition for mixin classes."""
        if self.already_defined:
            # pylint: disable=attribute-defined-outside-init
            self.already_defined = None


# Test for regression in bitbucket issue 164
# https://bitbucket.org/logilab/pylint/issue/164/
class MyClass1:
    def __init__(self):
        self.first += 5  # [access-member-before-definition]
        self.first = 0



"""Test the superfluous-parens warning."""
# pylint: disable=unneeded-not, unnecessary-comprehension, missing-function-docstring, invalid-name, fixme
# pylint: disable=import-error, missing-class-docstring, too-few-public-methods
import numpy as np
A = 3
if (A == 5):  # [superfluous-parens]
    pass
if not (A == 5):  # [superfluous-parens]
    pass
if not (3 or 5):
    pass
for (x) in (1, 2, 3):  # [superfluous-parens]
    print(x)
if (1) in (1, 2, 3):  # [superfluous-parens]
    pass
if (1, 2) in (1, 2, 3):
    pass
DICT = {'a': 1, 'b': 2}
del(DICT['b'])  # [superfluous-parens]
del DICT['a']

B = [x for x in ((3, 4))]
C = [x for x in ((3, 4) if 1 > 0 else (5, 6))]  # pylint: disable=comparison-of-constants
D = [x for x in ((3, 4) if 1 > 0 else ((5, 6)))]  # pylint: disable=comparison-of-constants
E = [x for x in ((3, 4) if 1 > 0 else ((((5, 6)))))]  # pylint: disable=comparison-of-constants

# Test assertions
F = "Version 1.0"
G = "1.0"
assert "Version " + G in F
assert ("Version " + G) in F # [superfluous-parens]

# Test assignment
H = 2 + (5 * 3)
NUMS_LIST = ['1', '2', '3']
NUMS_SET = {'1', '2', '3'}
NUMS_DICT = {'1': 1, '2': 2, '3': 3}
I = tuple(x for x in ((a, str(a)) for a in ()))

# Test functions
def function_A():
    return (x for x in ((3, 4)))

def function_B(var):
    return (var.startswith(('A', 'B', 'C')) or var == 'D')

def function_C(first, second):
    return (first or second) in (0, 1)

# Test numpy
def function_numpy_A(var_1: int, var_2: int) -> np.ndarray:
    result = (((var_1 & var_2)) > 0)
    return result.astype(np.float32)

def function_numpy_B(var_1: int, var_2: int) -> np.ndarray:
    return (((var_1 & var_2)) > 0).astype(np.float32)

# Test Class
class ClassA:
    keys = []

    def __iter__(self):
        return ((k, getattr(self, k)) for k in self.keys)

if (A == 2) is not (B == 2):
    pass

K = ("Test " + "String")  # [superfluous-parens]
M = A is not (A <= H)
M = True is not (M == K)
M = True is not (True is not False)  # pylint: disable=comparison-of-constants

Z = "TestString"
X = ("Test " + "String")  # [superfluous-parens]
Y = ("Test " + "String") in Z  # [superfluous-parens]
assert ("Test " + "String") in "hello"  # [superfluous-parens]
assert ("Version " + "String") in ("Version " + "String")  # [superfluous-parens]

hi = ("CONST")  # [superfluous-parens]
hi = ("CONST",)

#TODO: maybe get this line to report [superfluous-parens] without causing other false positives.
assert "" + ("Version " + "String") in Z



# pylint: disable=missing-docstring, unused-argument, wrong-import-position, invalid-name
from pathlib import Path

# Regression test for https://github.com/pylint-dev/pylint/issues/400
class TestListener:
    def __init__(self):
        self._latest = None

    def wait(self, timeout):
        result = self._latest
        self._latest = None
        return result

    def peer_joined(self, peer):
        self._latest = peer


listener = TestListener()
broker = listener.wait(3).get_domain()  # No error here


# Regression test for https://github.com/pylint-dev/pylint/issues/4377
from urllib import parse

url = 'http://www.google.com'

parsed_url = parse.urlparse(url)
sorted_query = parse.urlencode(
    sorted(parse.parse_qsl(parsed_url.query), key=lambda param: param[0]))
new_parsed_url = parse.ParseResult._replace(parsed_url, query=sorted_query)
new_url = new_parsed_url.geturl()  # No error here


# Regression test for https://github.com/pylint-dev/pylint/issues/3803
# pylint: disable=too-few-public-methods
class Base:
    label: str


class Derived(Base):
    label = "I exist!"


print(Derived.label)


# Regression test for https://github.com/pylint-dev/pylint/issues/5832
starter_path = Path(__file__).parents[3].resolve()



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from pylint.checkers import strings

TEST_TOKENS = (
    '"X"',
    "'X'",
    "'''X'''",
    '"""X"""',
    'r"X"',
    "R'X'",
    'u"X"',
    "F'X'",
    'f"X"',
    "F'X'",
    'fr"X"',
    'Fr"X"',
    'fR"X"',
    'FR"X"',
    'rf"X"',
    'rF"X"',
    'Rf"X"',
    'RF"X"',
)


def test_str_eval() -> None:
    for token in TEST_TOKENS:
        assert strings.str_eval(token) == "X"



"""Test that unused-import is not emitted here when everything else is disabled

https://github.com/pylint-dev/pylint/issues/3445
https://github.com/pylint-dev/pylint/issues/6089
"""
from math import e, pi
from os import environ

for k, v in environ.items():
    print(k, v)


class MyClass:
    """For the bug reported in #6089 it is important to use the same names for the class attributes as in the imports."""

    e = float(e)
    pi = pi



"""Tests for missing-param-doc and missing-type-doc for non-specified style docstrings
with accept-no-param-doc = no and the default value of no-docstring-rgx
"""
# pylint: disable=invalid-name, unused-argument


def _test_skip_no_docstring_rgx(x, y):
    """Example of a function that matches the default 'no-docstring-rgx' config option

    No error message is emitted.
    """



class Fruit:
    COLORS = []

    def __init__(self, color):
        self.color = color

    @classmethod
    def pick_colors(cls, *args):
        """classmethod to pick fruit colors"""
        cls.COLORS = args



# pylint: disable=missing-docstring

from typing import TypeAlias

def unused_variable_should_not_be_emitted():
    """unused-variable shouldn't be emitted for Example."""
    Example: TypeAlias = int
    result: set["Example"] = set()
    return result



# pylint: disable=missing-docstring, too-few-public-methods, expression-not-assigned, line-too-long

a_dict = {}
b_dict = {}

for k, v in a_dict.items():
    print(a_dict[k])  # [unnecessary-dict-index-lookup]
    print(b_dict[k])  # Should not emit warning, accessing other dictionary
    a_dict[k] = 123  # Should not emit warning, key access necessary
    a_dict[k] += 123  # Should not emit warning, key access necessary
    print(a_dict[k])  # Should not emit warning, v != a_dict[k]

for k, v in b_dict.items():
    print(k)
    k = "another key"
    print(b_dict[k])  # This is fine, key reassigned


# Tests on comprehensions
A = {v: 1 for k, v in a_dict.items() if a_dict[k]}  # [unnecessary-dict-index-lookup]
B = {v: 1 for k, v in a_dict.items() if k}  # This is fine, no indexing
C = {a_dict[k]: 1 for k, v in a_dict.items() if k}  # [unnecessary-dict-index-lookup]
# +1: [unnecessary-dict-index-lookup, unnecessary-dict-index-lookup]
D = {a_dict[k]: 1 for k, v in a_dict.items() if a_dict[k]}

E = [v for k, v in a_dict.items() if a_dict[k]]  # [unnecessary-dict-index-lookup]
F = [v for k, v in a_dict.items() if k]  # This is fine, no indexing
G = [a_dict[k] for k, v in a_dict.items() if k]  # [unnecessary-dict-index-lookup]
# +1: [unnecessary-dict-index-lookup, unnecessary-dict-index-lookup]
H = [a_dict[k] for k, v in a_dict.items() if a_dict[k]]


# Tests on dict attribute of a class
class Foo:
    c_dict = {}


for k, v in Foo.c_dict.items():
    print(b_dict[k])  # Should not emit warning, accessing other dictionary
    print(Foo.c_dict[k])  # [unnecessary-dict-index-lookup]
    unnecessary = 0  # pylint: disable=invalid-name
    unnecessary += Foo.c_dict[k]  # [unnecessary-dict-index-lookup]
    Foo.c_dict[k] += v  # key access necessary

# Tests on comprehensions
S = {v: 1 for k, v in Foo.c_dict.items() if Foo.c_dict[k]}  # [unnecessary-dict-index-lookup]
J = {v: 1 for k, v in Foo.c_dict.items() if k}  # This is fine, no indexing
K = {Foo.c_dict[k]: 1 for k, v in Foo.c_dict.items() if k}  # [unnecessary-dict-index-lookup]
# +1: [unnecessary-dict-index-lookup, unnecessary-dict-index-lookup]
L = {Foo.c_dict[k]: 1 for k, v in Foo.c_dict.items() if Foo.c_dict[k]}

M = [v for k, v in Foo.c_dict.items() if Foo.c_dict[k]]  # [unnecessary-dict-index-lookup]
N = [v for k, v in Foo.c_dict.items() if k]  # This is fine, no indexing
T = [Foo.c_dict[k] for k, v in Foo.c_dict.items() if k]  # [unnecessary-dict-index-lookup]
# +1: [unnecessary-dict-index-lookup, unnecessary-dict-index-lookup]
P = [Foo.c_dict[k] for k, v in Foo.c_dict.items() if Foo.c_dict[k]]

# Test assigning d.items() to a single variable
d = {1: "a", 2: "b"}
for item in d.items():
    print(item[0])
    print(d[item[0]])  # [unnecessary-dict-index-lookup]

Q = [item[0] for item in d.items()]
R = [d[item[0]] for item in d.items()]  # [unnecessary-dict-index-lookup]

# Reassigning single var
for item in d.items():
    print(item[0])
    print(d[item[0]])  # [unnecessary-dict-index-lookup]
    item = (2, "b")
    print(d[item[0]])  # This is fine, no warning thrown as key has been reassigned


# Test false positive described in #4630
# (https://github.com/pylint-dev/pylint/issues/4630)

d = {'key': 'value'}

for k, _ in d.items():
    d[k] += 'VALUE'
    if 'V' in d[k]:  # This is fine, if d[k] is replaced with _, the semantics change
        print('found V')


for k, _ in d.items():
    if 'V' in d[k]:  # [unnecessary-dict-index-lookup]
        d[k] = "value"
        print(d[k])  # This is fine

# Test false positive described in #4716
# Should not be emitted for del
# (https://github.com/pylint-dev/pylint/issues/4716)
d = {}
for key, val in d.items():
    del d[key]
    break

for item in d.items():
    del d[item[0]]
    break

outer_dict = {"inner_dict": {}}
for key, val in outer_dict.items():
    for key_two, val_two in val.items():
        del outer_dict[key][key_two]  # [unnecessary-dict-index-lookup]
        break

# Test partial unpacking of items
# https://github.com/pylint-dev/pylint/issues/5504

d = {}
for key, in d.items():
    print(d[key])

# Test subscripting an attribute
# https://github.com/pylint-dev/pylint/issues/6557
f = Foo()
for input_output in d.items():
    f.input_output = input_output  # pylint: disable=attribute-defined-outside-init
    print(d[f.input_output[0]])

# Regression test for https://github.com/pylint-dev/pylint/issues/6788
d = {'a': 1, 'b': 2, 'c': 3}
for key, val in d.items():
    ([d[key], x], y) = ([1, 2], 3)

# Regression test for https://github.com/pylint-dev/pylint/issues/6818
d = {'a': 1, 'b': 2, 'c': 3}
for key, val in d.items():
    while d[key] > 0:
        d[key] -= 1



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""This launches the configuration functional tests. This permits to test configuration
files by providing a file with the appropriate extension in the ``tests/config/functional``
directory.

Let's say you have a regression_list_crash.toml file to test. Then, if there is an error in the
conf, add ``regression_list_crash.out`` alongside your file with the expected output of pylint in
it. Use ``{relpath}`` and ``{abspath}`` for the path of the file. The exit code will have to be 2
 (error) if this file exists.

You must also define a ``regression_list_crash.result.json`` if you want to check the parsed
configuration. This file will be loaded as a dict and will override the default value of the
default pylint configuration. If you need to append or remove a value use the special key
``"functional_append"`` and ``"functional_remove":``. Check the existing code for examples.
"""

# pylint: disable=redefined-outer-name
import logging
import warnings
from pathlib import Path

import pytest
from pytest import CaptureFixture, LogCaptureFixture

from pylint.testutils.configuration_test import (
    PylintConfiguration,
    get_expected_configuration,
    get_expected_output,
    run_using_a_configuration_file,
)

HERE = Path(__file__).parent
USER_SPECIFIC_PATH = HERE.parent.parent
FUNCTIONAL_DIR = HERE / "functional"
# We use string then recast to path, so we can use -k in pytest.
# Otherwise, we get 'configuration_path0' as a test name. The path is relative to the functional
# directory because otherwise the string would be very lengthy.
ACCEPTED_CONFIGURATION_EXTENSIONS = ("toml", "ini", "cfg")
CONFIGURATION_PATHS = [
    str(path.relative_to(FUNCTIONAL_DIR))
    for ext in ACCEPTED_CONFIGURATION_EXTENSIONS
    for path in FUNCTIONAL_DIR.rglob(f"*.{ext}")
    if (str_path := str(path))
    # The enable/disable all tests are not practical with this framework.
    # They require manually listing ~400 messages, which will
    # require constant updates.
    and "enable_all" not in str_path and "disable_all" not in str_path
]


@pytest.fixture()
def default_configuration(
    tmp_path: Path, file_to_lint_path: str
) -> PylintConfiguration:
    empty_pylintrc = tmp_path / "pylintrc"
    empty_pylintrc.write_text("")
    mock_exit, _, runner = run_using_a_configuration_file(
        str(empty_pylintrc), file_to_lint_path
    )
    mock_exit.assert_called_once_with(0)
    return runner.linter.config.__dict__


@pytest.mark.parametrize("configuration_path", CONFIGURATION_PATHS)
def test_functional_config_loading(
    configuration_path: str,
    default_configuration: PylintConfiguration,
    file_to_lint_path: str,
    capsys: CaptureFixture[str],
    caplog: LogCaptureFixture,
) -> None:
    """Functional tests for configurations."""
    # logging is helpful to see what's expected and why. The output of the
    # program is checked during the test so printing messes with the result.
    caplog.set_level(logging.INFO)
    configuration_path = str(FUNCTIONAL_DIR / configuration_path)
    msg = f"Wrong result with configuration {configuration_path}"
    expected_code, expected_output = get_expected_output(
        configuration_path, USER_SPECIFIC_PATH
    )
    expected_loaded_configuration = get_expected_configuration(
        configuration_path, default_configuration
    )
    with warnings.catch_warnings():
        warnings.filterwarnings(
            "ignore", message="The use of 'MASTER'.*", category=UserWarning
        )
        mock_exit, _, runner = run_using_a_configuration_file(
            configuration_path, file_to_lint_path
        )
    mock_exit.assert_called_once_with(expected_code)
    out, err = capsys.readouterr()
    # 'rstrip()' applied, so we can have a final newline in the expected test file
    assert expected_output.rstrip() == out.rstrip(), msg
    assert sorted(expected_loaded_configuration.keys()) == sorted(
        runner.linter.config.__dict__.keys()
    ), msg
    for key, expected_value in expected_loaded_configuration.items():
        key_msg = f"{msg} for key '{key}':"
        if isinstance(expected_value, list):
            assert sorted(expected_value) == sorted(
                runner.linter.config.__dict__[key]
            ), key_msg
        else:
            assert expected_value == runner.linter.config.__dict__[key], key_msg
    assert not err, msg



# pylint: disable=consider-alternative-union-syntax
from __future__ import annotations

from typing import Optional, Union


class Dummy:
    pass


class Dummy2:
    class_attr: list[Dummy] = []

    def __init__(self, param: str) -> None:
        self.param = param
        self.union: Union[int, str] = ""
        self.alternative_union_syntax: str | int = 0
        self.optional: Optional[Dummy] = None
        self.alternative_optional: int | None = None
        self.alternative_optional_swapped: None | int = None
        self.optional_union: int | str = None



"""Test case for TypeVar name not matching assigned variable name."""
from typing import TypeVar

# Control examples
T = TypeVar("T")
T_co = TypeVar("T_co", covariant=True)
T_contra = TypeVar("T_contra", contravariant=True)

# Mismatching names
X = TypeVar("T")  # [typevar-name-mismatch]
X_co = TypeVar("T_co", covariant=True)  # [typevar-name-mismatch]
X_contra = TypeVar("T_contra", contravariant=True)  # [typevar-name-mismatch]
X_co, X_contra = (  # [typevar-name-mismatch,typevar-name-mismatch]
    TypeVar("T", covariant=True),
    TypeVar("T", contravariant=True),
)

# The user may also violate typevar-double-variance
# or typevar-name-incorrect-variance
# on top of not matching the variable and TypeVar names.
# This rule does not suggest what the correct name is
# (that's already handled by the aforementioned rules),
# it just highlights that the names don't match.
X = TypeVar("T", contravariant=True)  # [typevar-name-mismatch,typevar-name-incorrect-variance]
X = TypeVar(  # [typevar-name-mismatch,typevar-name-incorrect-variance,typevar-double-variance]
    "T",
    covariant=True,
    contravariant=True
)
X_co = TypeVar("T_co")  # [typevar-name-mismatch,typevar-name-incorrect-variance]
X_contra = TypeVar(  # [typevar-name-mismatch,typevar-name-incorrect-variance]
    "T_contra",
    covariant=True
)

# name can be specified as a keyword argument as well.
T = TypeVar(name="T")
T_co = TypeVar(name="T_co", covariant=True)
T_contra = TypeVar(name="T_contra", contravariant=True)
T_co = TypeVar(covariant=True, name="T_co")
T_contra = TypeVar(contravariant=True, name="T_contra")

X = TypeVar(name="T")  # [typevar-name-mismatch]
X_co = TypeVar(name="T_co", covariant=True)  # [typevar-name-mismatch]
X_contra = TypeVar(name="T_contra", contravariant=True) # [typevar-name-mismatch]
X_co = TypeVar(covariant=True, name="T_co")  # [typevar-name-mismatch]
X_contra = TypeVar(contravariant=True, name="T_contra") # [typevar-name-mismatch]



list([0 for y in list(range(10))])  # [consider-using-generator]
tuple([0 for y in list(range(10))])  # [consider-using-generator]
sum([y**2 for y in list(range(10))])  # [consider-using-generator]
max([y**2 for y in list(range(10))])  # [consider-using-generator]
min([y**2 for y in list(range(10))])  # [consider-using-generator]



# pylint: disable=too-few-public-methods,unused-argument
"""tagging a function as a class method cause a crash when checking for
signature overriding
"""

def fetch_config(mainattr=None):
    """return a class method"""

    def fetch_order(cls, attr, var):
        """a class method"""
        if attr == mainattr:
            return var
        return None
    fetch_order = classmethod(fetch_order)
    return fetch_order

class Aaa:
    """hop"""
    fetch_order = fetch_config('A')



# pylint: disable=invalid-name, undefined-variable, unused-variable, missing-function-docstring, missing-module-docstring
# pylint: disable=unsupported-assignment-operation, line-too-long

if f():  # [consider-ternary-expression]
    x = 4
else:
    x = 5

if g():
    y = 3
elif h():
    y = 4
else:
    y = 5

def a():
    if i():  # [consider-ternary-expression]
        z = 4
    else:
        z = 5

if f():
    x = 4
    print(x)
else:
    x = 5

if f():
    x[0] = 4
else:
    x = 5

if f():
    x = 4
else:
    y = 5



# pylint: disable=missing-function-docstring, missing-module-docstring, invalid-name, import-outside-toplevel
import codecs
import contextlib
import multiprocessing
import pathlib
import subprocess
import tarfile
import tempfile
import threading
import urllib
import zipfile
from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
from pathlib import Path


def test_pathlib_open():
    _ = pathlib.Path("foo").open(encoding="utf8")  # [consider-using-with]
    _ = Path("foo").open(encoding="utf8")  # [consider-using-with]
    path = Path("foo")
    _ = path.open(encoding="utf8")  # [consider-using-with]
    with Path("foo").open(encoding="utf8") as file:  # must not trigger
        _ = file.read()


def test_codecs_open():
    fh = codecs.open("test.txt", "utf8")  # [consider-using-with]
    fh.close()


def test_urlopen():
    _ = urllib.request.urlopen("http://www.python.org")  # [consider-using-with]


def test_temporary_file():
    _ = tempfile.TemporaryFile("r")  # ambiguous with NamedTemporaryFile


def test_named_temporary_file():
    _ = tempfile.NamedTemporaryFile("r")  # [consider-using-with]


def test_spooled_temporary_file():
    _ = tempfile.SpooledTemporaryFile("r")  # [consider-using-with]


def test_temporary_directory():
    _ = tempfile.TemporaryDirectory()  # [consider-using-with]


def test_zipfile():
    myzip = zipfile.ZipFile("spam.zip", "w")  # [consider-using-with]
    _ = myzip.open("eggs.txt")  # [consider-using-with]


def test_pyzipfile():
    myzip = zipfile.PyZipFile("spam.zip", "w")  # [consider-using-with]

    with zipfile.PyZipFile("spam.zip", "w"):  # must not trigger
        pass

    _ = myzip.open("eggs.txt")  # [consider-using-with]

    with myzip.open("eggs.txt"):  # must not trigger
        pass


def test_tarfile():
    tf = tarfile.open("/tmp/test.tar", "w")  # [consider-using-with]
    tf.close()

    with tarfile.open("/tmp/test.tar", "w"):  # must not trigger
        pass

    tf = tarfile.TarFile("/tmp/test2.tar", "w")  # [consider-using-with]
    tf.close()

    with tarfile.TarFile("/tmp/test2.tar", "w"):  # must not trigger
        pass


def test_lock_acquisition():
    lock = threading.Lock()
    lock.acquire()  # [consider-using-with]
    lock.release()

    with lock:  # must not trigger
        pass

    rlock = threading.RLock()
    rlock.acquire()  # [consider-using-with]
    rlock.release()

    with rlock:  # must not trigger
        pass

    sema = threading.Semaphore()
    sema.acquire()  # [consider-using-with]
    sema.release()

    with sema:  # must not trigger
        pass

    bounded_sema = threading.BoundedSemaphore()
    bounded_sema.acquire()  # [consider-using-with]
    bounded_sema.release()

    with bounded_sema:  # must not trigger
        pass


@contextlib.contextmanager
def test_lock_acquisition_in_context_manager1():
    """
    The message must not be triggered if the resource allocation is done inside a context manager.
    """
    lock = threading.Lock()
    lock.acquire()  # must not trigger
    yield
    lock.release()


class MyLockContext:
    """
    The message must not be triggered if the resource allocation is done inside a context manager.
    """

    def __init__(self):
        self.lock = threading.Lock()

    def __enter__(self):
        self.lock.acquire()  # must not trigger

    def __exit__(self, exc_type, exc_value, traceback):
        self.lock.release()


def test_multiprocessing():
    # the different Locks provided by multiprocessing would be candidates
    # for consider-using-with as well, but they lead to InferenceErrors.
    _ = multiprocessing.Pool()  # [consider-using-with]
    with multiprocessing.Pool():
        pass

    manager = multiprocessing.managers.BaseManager()
    manager.start()  # [consider-using-with]
    with multiprocessing.managers.BaseManager():
        pass

    manager = multiprocessing.managers.SyncManager()
    manager.start()  # [consider-using-with]
    with multiprocessing.managers.SyncManager():
        pass


def test_popen():
    _ = subprocess.Popen("sh")  # [consider-using-with]
    with subprocess.Popen("sh"):
        pass


def test_suppress_in_exit_stack():
    """Regression test for issue #4654 (false positive)"""
    with contextlib.ExitStack() as stack:
        _ = stack.enter_context(
            open("/sys/firmware/devicetree/base/hwid,location", "r", encoding="utf-8")
        )  # must not trigger


def test_futures():
    """
    Regression test for issue #4689.
    ThreadPoolExecutor and ProcessPoolExecutor were formerly part of the callables that raised
    the R1732 message if used outside a with block, but there are legitimate use cases where
    Executor instances are used e.g. as a persistent background worker pool throughout the program.
    """
    thread_executor = ThreadPoolExecutor()
    thread_executor.submit(print, 1)
    process_executor = ProcessPoolExecutor()
    process_executor.submit(print, 2)
    thread_executor.shutdown()
    process_executor.shutdown()


pool = multiprocessing.Pool()  # must not trigger, as it is used later on
with pool:
    pass


global_pool = multiprocessing.Pool()  # must not trigger, will be used in nested scope


def my_nested_function():
    with global_pool:
        pass


# this must also work for tuple unpacking
pool1, pool2 = (
    multiprocessing.Pool(),  # must not trigger
    multiprocessing.Pool(),  # must not trigger
)

with pool1:
    pass

with pool2:
    pass

unused_pool1, unused_pool2 = (
    multiprocessing.Pool(),  # [consider-using-with]
    multiprocessing.Pool(),  # [consider-using-with]
)

used_pool, unused_pool = (
    multiprocessing.Pool(),  # must not trigger
    multiprocessing.Pool(),  # [consider-using-with]
)
with used_pool:
    pass

unused_pool, used_pool = (
    multiprocessing.Pool(),  # [consider-using-with]
    multiprocessing.Pool(),  # must not trigger
)
with used_pool:
    pass


def test_subscript_assignment():
    """
    Regression test for issue https://github.com/pylint-dev/pylint/issues/4732.
    If a context manager is assigned to a list or dict, we are not able to
    tell if / how the context manager is used later on, as it is not assigned
    to a variable or attribute directly.
    In this case we can only emit the message directly.
    """
    job_list = [None, None]
    job_list[0] = subprocess.Popen("ls")  # [consider-using-with]
    job_dict = {}
    job_dict["myjob"] = subprocess.Popen("ls")  # [consider-using-with]



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""For the visitors.diadefs module."""

# pylint: disable=redefined-outer-name

from __future__ import annotations

import os
import warnings
from collections.abc import Generator
from pathlib import Path

import astroid
import pytest

from pylint.pyreverse import inspector
from pylint.pyreverse.inspector import Project
from pylint.testutils.utils import _test_cwd
from pylint.typing import GetProjectCallable

HERE = Path(__file__)
TESTS = HERE.parent.parent


@pytest.fixture
def project(get_project: GetProjectCallable) -> Generator[Project, None, None]:
    with _test_cwd(TESTS):
        project = get_project("data", "data")
        linker = inspector.Linker(project)
        with warnings.catch_warnings():
            warnings.filterwarnings("ignore", category=DeprecationWarning)
            linker.visit(project)
        yield project


def test_locals_assignment_resolution(project: Project) -> None:
    klass = project.get_module("data.clientmodule_test")["Specialization"]
    assert hasattr(klass, "locals_type")
    type_dict = klass.locals_type
    assert len(type_dict) == 2
    keys = sorted(type_dict.keys())
    assert keys == ["TYPE", "top"]
    assert len(type_dict["TYPE"]) == 1
    assert type_dict["TYPE"][0].value == "final class"
    assert len(type_dict["top"]) == 1
    assert type_dict["top"][0].value == "class"


def test_instance_attrs_resolution(project: Project) -> None:
    klass = project.get_module("data.clientmodule_test")["Specialization"]
    assert hasattr(klass, "instance_attrs_type")
    type_dict = klass.instance_attrs_type
    assert len(type_dict) == 3
    keys = sorted(type_dict.keys())
    assert keys == ["_id", "relation", "relation2"]
    assert isinstance(type_dict["relation"][0], astroid.bases.Instance), type_dict[
        "relation"
    ]
    assert type_dict["relation"][0].name == "DoNothing"
    assert type_dict["_id"][0] is astroid.Uninferable


def test_from_directory(project: Project) -> None:
    expected = os.path.join("tests", "data", "__init__.py")
    assert project.name == "data"
    assert project.path.endswith(expected)


def test_project_node(project: Project) -> None:
    expected = [
        "data",
        "data.clientmodule_test",
        "data.nullable_pattern",
        "data.property_pattern",
        "data.suppliermodule_test",
    ]
    assert sorted(project.keys()) == expected



"""Tests for missing-raises-doc and missing-raises-type-doc for Sphinx style docstrings
with accept-no-raise-doc = no"""
# pylint: disable=function-redefined, invalid-name, undefined-variable, missing-function-docstring
# pylint: disable=unused-argument, import-outside-toplevel


def test_sphinx_raises_with_prefix_one(self):
    """This is a sphinx docstring.

    :raises ~re.error: Sometimes
    """
    import re

    raise re.error("hi")


def test_sphinx_raises_with_prefix_two(self):
    """This is a sphinx docstring.

    :raises !re.error: Sometimes
    """
    import re

    raise re.error("hi")



from pylint.checkers.exceptions import ExceptionsChecker
from pylint.checkers.stdlib import StdlibChecker
from pylint.extensions.check_elif import ElseifUsedChecker


class CheckerCollector:
    def __init__(self):
        self.checker1 = ExceptionsChecker(None)
        self.checker2 = ElseifUsedChecker(None)
        self.checker3 = StdlibChecker(None)



"""Test to see we don't crash on this code in pandas.
See: https://github.com/pandas-dev/pandas/blob/main/pandas/core/arrays/sparse/array.py
Code written by Guido van Rossum here: https://github.com/python/typing/issues/684"""
# pylint: disable=no-member, redefined-builtin, invalid-name, missing-class-docstring

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from enum import Enum

    class ellipsis(Enum):
        Ellipsis = "..."

    Ellipsis = ellipsis.Ellipsis


else:
    ellipsis = type(Ellipsis)



"""Test for a regression on slots and annotated assignments.
Reported in https://github.com/pylint-dev/pylint/issues/5479
"""
# pylint: disable=too-few-public-methods, unused-private-member, missing-class-docstring, missing-function-docstring

from __future__ import annotations

import asyncio


class Connector:
    __slots__ = ("_Connector__reader", "_Connector__writer")

    __reader: asyncio.StreamReader
    __writer: asyncio.StreamWriter

    def __init__(self) -> None:
        raise TypeError("Use connect() instead")

    @classmethod
    async def connect(cls, socket: str) -> Connector:
        self = cls.__new__(cls)
        self.__reader, self.__writer = await asyncio.open_unix_connection(socket)
        return self


async def main():
    conn = await Connector.connect("/tmp/mysocket")  # [unused-variable]


asyncio.run(main())



# Test for https://github.com/pylint-dev/pylint/issues/9045

class P:
    pass

class A:
    x: P

class B:
    def __init__(self, x: P):
        self.x = x

class C:
    x: P

    def __init__(self, x: P):
        self.x = x

class D:
    x: P

    def __init__(self):
        self.x = P()

class E:
    def __init__(self):
        self.x = P()



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

import os
from pathlib import Path
from typing import Any, NoReturn
from unittest import mock
from unittest.mock import patch

from pytest import CaptureFixture

from pylint.lint.pylinter import MANAGER, PyLinter
from pylint.utils import FileState


def raise_exception(*args: Any, **kwargs: Any) -> NoReturn:
    raise ValueError


@patch.object(FileState, "iter_spurious_suppression_messages", raise_exception)
def test_crash_in_file(
    linter: PyLinter, capsys: CaptureFixture[str], tmp_path: Path
) -> None:
    linter.crash_file_path = str(tmp_path / "pylint-crash-%Y")
    linter.check([__file__])
    out, err = capsys.readouterr()
    assert not out
    assert not err
    files = os.listdir(tmp_path)
    assert len(files) == 1
    assert "pylint-crash-20" in str(files[0])
    assert any(m.symbol == "fatal" for m in linter.reporter.messages)


def test_crash_during_linting(
    linter: PyLinter, capsys: CaptureFixture[str], tmp_path: Path
) -> None:
    with mock.patch(
        "pylint.lint.PyLinter.check_astroid_module", side_effect=RuntimeError
    ):
        linter.crash_file_path = str(tmp_path / "pylint-crash-%Y")
        linter.check([__file__])
        out, err = capsys.readouterr()
        assert not out
        assert not err
        files = os.listdir(tmp_path)
        assert len(files) == 1
        assert "pylint-crash-20" in str(files[0])
        assert any(m.symbol == "astroid-error" for m in linter.reporter.messages)


def test_open_pylinter_denied_modules(linter: PyLinter) -> None:
    """Test PyLinter open() adds ignored modules to Astroid manager deny list."""
    MANAGER.module_denylist = {"mod1"}
    try:
        linter.config.ignored_modules = ["mod2", "mod3"]
        linter.open()
        assert MANAGER.module_denylist == {"mod1", "mod2", "mod3"}
    finally:
        MANAGER.module_denylist = set()


def test_open_pylinter_prefer_stubs(linter: PyLinter) -> None:
    try:
        linter.config.prefer_stubs = True
        linter.open()
        assert MANAGER.prefer_stubs
    finally:
        MANAGER.prefer_stubs = False



def divide_x_by_y(x: float, y: float):
    try:
        print(x / y)
    except FloatingPointError as e:
        print(f"There was a FloatingPointError: {e}")
    except ArithmeticError as e:
        # FloatingPointError  were already caught at this point
        print(f"There was an OverflowError or a ZeroDivisionError: {e}")



"""Check invalid value returned by __index__ """

# pylint: disable=too-few-public-methods,missing-docstring,import-error,unnecessary-lambda-assignment
import six

from missing import Missing


class FirstGoodIndex:
    """__index__ returns <type 'int'>"""

    def __index__(self):
        return 1


class SecondGoodIndex:
    """__index__ returns <type 'int'>"""

    def __index__(self):
        return 0


class IndexMetaclass(type):
    def __index__(cls):
        return 1


@six.add_metaclass(IndexMetaclass)
class ThirdGoodIndex:
    """Index through the metaclass."""


class FirstBadIndex:
    """ __index__ returns a dict """

    def __index__(self):  # [invalid-index-returned]
        return {'1': '1'}


class SecondBadIndex:
    """ __index__ returns str """

    def __index__(self):  # [invalid-index-returned]
        return "42"


class ThirdBadIndex:
    """ __index__ returns a float"""

    def __index__(self):  # [invalid-index-returned]
        return 1.11


class FourthBadIndex:
    """ __index__ returns node which does not have 'value' in AST """

    def __index__(self):  # [invalid-index-returned]
        return lambda: 3


class AmbigousIndex:
    """ Uninferable return value """

    __index__ = lambda self: Missing


class AnotherAmbiguousIndex:
    """Potential uninferable return value"""

    def __index__(self):
        return int(Missing)



class Fruit:
    COLORS = []

    def __init__(self, color):
        self.color = color

    def pick_colors(cls, *args):
        """classmethod to pick fruit colors"""
        cls.COLORS = args

    pick_colors = classmethod(pick_colors)  # [no-classmethod-decorator]



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from __future__ import annotations

import pytest
from astroid import nodes

from pylint.pyreverse.dot_printer import DotPrinter
from pylint.pyreverse.plantuml_printer import PlantUmlPrinter
from pylint.pyreverse.printer import Layout, NodeType, Printer


@pytest.mark.parametrize(
    "layout, printer_class, expected_content, line_index",
    [
        (Layout.TOP_TO_BOTTOM, DotPrinter, "rankdir=TB", -2),
        (Layout.BOTTOM_TO_TOP, DotPrinter, "rankdir=BT", -2),
        (Layout.LEFT_TO_RIGHT, DotPrinter, "rankdir=LR", -2),
        (Layout.RIGHT_TO_LEFT, DotPrinter, "rankdir=RL", -2),
        (Layout.TOP_TO_BOTTOM, PlantUmlPrinter, "top to bottom direction", -1),
        (Layout.LEFT_TO_RIGHT, PlantUmlPrinter, "left to right direction", -1),
    ],
)
def test_explicit_layout(
    layout: Layout, printer_class: type[Printer], expected_content: str, line_index: int
) -> None:
    printer = printer_class(title="unittest", layout=layout)
    assert printer.lines[line_index].strip() == expected_content


@pytest.mark.parametrize(
    "layout, printer_class",
    [(Layout.BOTTOM_TO_TOP, PlantUmlPrinter), (Layout.RIGHT_TO_LEFT, PlantUmlPrinter)],
)
def test_unsupported_layout(layout: Layout, printer_class: type[Printer]) -> None:
    with pytest.raises(ValueError):
        printer_class(title="unittest", layout=layout)


def test_method_arguments_none() -> None:
    func = nodes.FunctionDef(
        name="func",
        lineno=1,
        col_offset=0,
        end_lineno=None,
        end_col_offset=None,
        parent=None,
    )
    args = nodes.Arguments(vararg=None, kwarg=None, parent=func)
    args.args = None
    func.postinit(args, body=None)
    parsed_args = Printer._get_method_arguments(func)
    assert parsed_args == []


class TestPlantUmlPrinter:
    printer = PlantUmlPrinter(title="unittest", layout=Layout.TOP_TO_BOTTOM)

    def test_node_without_properties(self) -> None:
        self.printer.emit_node(name="test", type_=NodeType.CLASS)
        assert self.printer.lines[-2:] == ['class "test" as test {\n', "}\n"]



import random

measurement = random.randint(0, 200)
above_threshold = False
i = 0
while i < 5:  # [magic-value-comparison]
    above_threshold = measurement > 100  # [magic-value-comparison]
    if above_threshold:
        break
    measurement = random.randint(0, 200)



def can_be_divided_by_two_and_are_not_zero(x, y, z):
    # Maximum number of boolean expressions in an if statement (by default 5)
    # +1: [too-many-boolean-expressions]
    if (x and y and z) and (x % 2 == 0 and y % 2 == 0 and z % 2 == 0):
        pass



def fifty_percent_off(whole):
    return (float(whole)) * 50 / 100


def calculate_sum_and_display_price_of_fruits(*fruits):  # [too-complex]
    # McCabe rating is 13 here (by default 10)
    shopping_list = []

    if "apple" in fruits:
        v = fifty_percent_off(1.1)
        shopping_list.append(v)
    if "pear" in fruits:
        shopping_list.append(0.8)
    if "banana" in fruits:
        shopping_list.append(1.2)
    if "mango" in fruits:
        shopping_list.append(3.5)
    if "peach" in fruits:
        shopping_list.append(0.5)
    if "melon" in fruits:
        shopping_list.append(4.9)
    if "orange" in fruits:
        shopping_list.append(2.0)
    if "strawberry" in fruits:
        shopping_list.append(2.5)
    if "mandarin" in fruits:
        shopping_list.append(2.3)
    if "plum" in fruits:
        shopping_list.append(0.5)
    if "watermelon" in fruits:
        v = fifty_percent_off(6.4)
        shopping_list.append(v)

    combine = zip(fruits, shopping_list)

    for i in combine:
        print(f"{i[0]} ${i[1]:.2f}")

    total = sum(shopping_list)
    print(f"Total price is ${total:.2f}")


fruits_to_buy = ["apple", "orange", "watermelon"]
calculate_sum_and_display_price_of_fruits(*fruits_to_buy)



"""Tests for logging-too-few-args"""

import logging

logging.error("{}, {}", 1)  # [logging-too-few-args]
logging.error("{0}, {1}", 1)  # [logging-too-few-args]
logging.error("{named1}, {named2}", {"named1": 1})  # [logging-too-few-args]
logging.error("{0}, {named}", 1)  # [logging-too-few-args]
logging.error("{}, {named}", {"named": 1})  # [logging-too-few-args]
logging.error("{0}, {named}", {"named": 1})  # [logging-too-few-args]



# pylint: disable=missing-docstring,too-few-public-methods,bad-option-value


def myfunc(): # [useless-return]
    print('---- testing ---')
    return

class SomeClass:
    def mymethod(self): # [useless-return]
        print('---- testing ---')
        return None

    # These are not emitted
    def item_at(self):
        return None


def function2(parameter):  # [useless-return]
    if parameter:
        pass
    return


def function3(parameter):  # [useless-return]
    if parameter:
        pass
    else:
        return


def function4(parameter):  # [useless-return]
    try:
        parameter.do()
    except RuntimeError:
        parameter.other()
        return


def function5(parameter):  # [useless-return]
    try:
        parameter.do()
    except RuntimeError:
        return


def code_after_return(param):
    try:
        param.kaboom()
    except RuntimeError:
        param.other()
        return

    param.something_else()
    param.state = "good"


def code_after_else(obj):
    if obj.k:
        pass
    else:
        return

    obj.do()


def return_in_loop(obj):
    for _ in range(10):
        obj.do()
        if obj.k:
            return

    return



"""Tests for except handlers that shadow outer except handlers or exceptions.

See: https://github.com/pylint-dev/pylint/issues/5370
"""

try:
    pass
except ImportError as err:
    try:
        pass
    except ImportError as err:  # [redefined-outer-name]
        pass
    print(err)

try:
    pass
except ImportError as err:
    try:
        pass
    except ImportError as err2:
        pass
    print(err)

try:
    try:
        pass
    except ImportError as err:
        pass
except ImportError:
    try:
        pass
    except ImportError as err:
        pass
    print(err)


try:
    try:
        pass
    except ImportError as err:
        pass
except ImportError as err:
    try:
        pass
    except ImportError:
        pass
    print(err)

try:
    pass
except ImportError as err:
    try:
        pass
    except ImportError as err2:
        try:
            pass
        except ImportError as err:  # [redefined-outer-name]
            pass
    print(err)


class CustomException(Exception):
    """https://github.com/pylint-dev/pylint/issues/4434"""


def func():
    """Override CustomException by except .. as .."""
    try:
        raise CustomException('Test')  # [used-before-assignment]
    # pylint:disable-next=invalid-name, unused-variable
    except IOError as CustomException:  # [redefined-outer-name]
        pass


# https://github.com/pylint-dev/pylint/issues/9671
def function_before_exception():
    """The local variable `e` should not trigger `redefined-outer-name`
       when `e` is also defined in the subsequent exception handling block.
    """
    e = 42
    return e

try:
    raise ValueError('outer')
except ValueError as e:
    print(e)


def function_after_exception():
    """The local variable `e` should not trigger `redefined-outer-name`
       when `e` is also defined in the preceding exception handling block.
    """
    e = 42
    return e



try:
    print(int(input()))
except Exception:
    raise
except TypeError:  # [bad-except-order]
    # This block cannot be reached since TypeError exception
    # is caught by previous exception handler.
    raise



"""Regression test for https://github.com/pylint-dev/pylint/issues/4899"""

# pylint: disable=missing-docstring,too-few-public-methods

from dataclasses import field
from typing import List
from pydantic.dataclasses import dataclass # [import-error]


class Item:
    pass


@dataclass
class Case:
    """Case class (group Item)"""

    name: str
    irr: float = 0
    items: List[Item] = field(default_factory=lambda: [])

    def add_item(self, item: Item) -> None:
        """Add an item to the item list."""

        self.items.append(item)

    def find_item(self, description: str) -> Item:
        """Find an item by description"""

        return next(
            (item for item in self.items if item.description == description), None
        )



NUMBERS_TO_STRINGS = {
    1: "one",
    2: "two",
    3: "three",
    4: "four",
    5: "five",
    6: "six",
    7: "seven",
}


def to_string(x):
    return f"This is {NUMBERS_TO_STRINGS.get(x)}."



import functools


class Fibonnaci:
    def __init__(self):
        self.result = []

    @functools.lru_cache(maxsize=None)  # [method-cache-max-size-none]
    def fibonacci(self, n):
        if n in {0, 1}:
            self.result.append(n)
        self.result.append(self.fibonacci(n - 1) + self.fibonacci(n - 2))



"""Attributes supplied by a decorator."""

from functools import lru_cache


class SomeClass:  # pylint: disable=too-few-public-methods
    """https://github.com/pylint-dev/pylint/issues/9246"""
    @classmethod
    @lru_cache
    def __cached_fun(cls, arg: int) -> str:
        return str(arg)

    @classmethod
    def cache_clear(cls):
        """__cached_fun()'s @cache decorator supplies cache_clear()."""
        cls.__cached_fun.cache_clear()



"""Regression test for trailing-whitespace (C0303)."""
# pylint: disable=mixed-line-endings,pointless-string-statement

# +1: [trailing-whitespace]
print('some trailing whitespace')   
# +1: [trailing-whitespace]
print('trailing whitespace does not count towards the line length limit')                   
print('windows line ends are ok')
# +1: [trailing-whitespace]
print('but trailing whitespace on win is not')   

# Regression test for https://github.com/pylint-dev/pylint/issues/6936
# +2: [trailing-whitespace]
""" This module has the Board class.
""" 

# +3: [trailing-whitespace]
""" This module has the Board class.
It's a very nice Board.
""" 

# Regression test for https://github.com/pylint-dev/pylint/issues/3822
def example(*args):
    """Example function."""
    print(*args)


example(
    "bob", """
    foobar 
    more text
""",
)

example(
    "bob",
    """
    foobar2 
    more text
""",
)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from pylint.lint import PyLinter
from pylint.message._deleted_message_ids import DELETED_MESSAGES_IDS


def test_no_removed_msgid_or_symbol_used(linter: PyLinter) -> None:
    """Tests that we're not using deleted msgid or symbol.

    This could cause occasional bugs, but more importantly confusion and inconsistencies
    when searching for old msgids online. See https://github.com/pylint-dev/pylint/issues/5729
    """
    for deleted_messages in DELETED_MESSAGES_IDS.values():
        for msgid, symbol, old_names in deleted_messages:
            linter.msgs_store.message_id_store.register_message_definition(
                msgid, symbol, old_names
            )



# pylint: disable=missing-docstring,too-few-public-methods,wrong-import-position,use-dict-literal
# pylint: disable=wrong-import-order, undefined-variable

REVISION = None

REVISION() # [not-callable]

def correct():
    return 1

REVISION = correct()

class Correct:
    """callable object"""

class MetaCorrect:
    """callable object"""
    def __call__(self):
        return self

INSTANCE = Correct()
CALLABLE_INSTANCE = MetaCorrect()
CORRECT = CALLABLE_INSTANCE()
INCORRECT = INSTANCE() # [not-callable]
LIST = []
INCORRECT = LIST() # [not-callable]
DICT = {}
INCORRECT = DICT() # [not-callable]
TUPLE = ()
INCORRECT = TUPLE() # [not-callable]
INT = 1
INCORRECT = INT() # [not-callable]

# Test calling properties. Pylint can detect when using only the
# getter, but it doesn't infer properly when having a getter
# and a setter.
class MyProperty(property):
    """ test subclasses """

class PropertyTest:
    """ class """

    def __init__(self):
        self.attr = 4

    @property
    def test(self):
        """ Get the attribute """
        return self.attr

    @test.setter
    def test(self, value):
        """ Set the attribute """
        self.attr = value

    @MyProperty
    def custom(self):
        """ Get the attribute """
        return self.attr

    @custom.setter
    def custom(self, value):
        """ Set the attribute """
        self.attr = value

PROP = PropertyTest()
PROP.test(40) # [not-callable]
PROP.custom() # [not-callable]

# Safe from not-callable when using properties.

class SafeProperty:
    @property
    def static(self):
        return staticmethod

    @property
    def klass(self):
        return classmethod

    @property
    def get_lambda(self):
        return lambda: None

    @property
    def other_function(self):
        def function(arg):
            return arg
        return function

    @property
    def dict_builtin(self):
        return dict

    @property
    def range_builtin(self):
        return range

    @property
    def instance(self):
        class Empty:
            def __call__(self):
                return 42
        return Empty()

    @property
    def does_not_make_sense(self):
        raise NotImplementedError

PROP1 = SafeProperty()
PROP1.static(2)
PROP1.klass(2)
PROP1.get_lambda()
PROP1.other_function(4)
PROP1.dict_builtin()
PROP1.range_builtin(4)
PROP1.instance()
PROP1.does_not_make_sense()


import missing  # pylint: disable=import-error


class UnknownBaseCallable(missing.Blah):
    pass

UnknownBaseCallable()()

# Regression test for #4426
# If property is inferable we shouldn't double emit the message
# See: https://github.com/pylint-dev/pylint/issues/4426
class ClassWithProperty:
    @property
    def value(self):
        return 42

CLASS_WITH_PROP = ClassWithProperty().value()  # [not-callable]

# Test typing.Namedtuple is callable
# See: https://github.com/pylint-dev/pylint/issues/1295
import typing

Named = typing.NamedTuple("Named", [("foo", int), ("bar", int)])
named = Named(1, 2)


# NamedTuple is callable, even if it aliased to an attribute
# See https://github.com/pylint-dev/pylint/issues/1730
class TestNamedTuple:
    def __init__(self, field: str) -> None:
        self.my_tuple = typing.NamedTuple("Tuple", [(field, int)])
        self.item: self.my_tuple

    def set_item(self, item: int) -> None:
        self.item = self.my_tuple(item)


# Test descriptor call
def func():
    pass


class ADescriptor:
    def __get__(self, instance, owner):
        return func


class AggregateCls:
    a = ADescriptor()


AggregateCls().a()


# Make sure not-callable isn't raised for descriptors

# astroid can't process descriptors correctly so
# pylint needs to ignore not-callable for them
# right now

# Test for https://github.com/pylint-dev/pylint/issues/1699

import multiprocessing

multiprocessing.current_process()

# Make sure not-callable isn't raised for uninferable properties
class MyClass:
    @property
    def call(self):
        return undefined


a = A()
a.call()

# Make sure the callable check does not crash when a node's parent cannot be determined.
def get_number(arg):
    return 2 * arg


get_number(10)()  # [not-callable]

class Klass:
    def __init__(self):
        self._x = None

    @property
    def myproperty(self):
        if self._x is None:
            self._x = lambda: None
        return self._x

myobject = Klass()
myobject.myproperty()

class Klass2:
    @property
    def something(self):
        if __file__.startswith('s'):
            return str

        return 'abcd'

obj2 = Klass2()
obj2.something()


# Regression test for https://github.com/pylint-dev/pylint/issues/7109
instance_or_cls = MyClass  # pylint:disable=invalid-name
instance_or_cls = MyClass()
if not isinstance(instance_or_cls, MyClass):
    new = MyClass.__new__(instance_or_cls)
    new()


# Regression test for https://github.com/pylint-dev/pylint/issues/5113.
# Do not emit `not-callable`.
ATTRIBUTES = {
    'DOMAIN': ("domain", str),
    'IMAGE': ("image", bytes),
}

for key, (name, validate) in ATTRIBUTES.items():
    name = validate(1)



class Fruit:
    def __init__(self):
        self.remaining_bites = 3

    def take_bite(self):
        if self.remaining_bites > 0:
            print(f"You take a bite of the {self.__class__.__name__.lower()}.")
            self.remaining_bites -= 1
        else:
            print(f"The {self.__class__.__name__.lower()} is already eaten up!")

    def eaten_by_animal(self, animal):
        self.remaining_bites = 0
        print(f"The {self.__class__.__name__.lower()} has been eaten by an animal.")



from dataclasses import dataclass, field


@dataclass
class C:
    a: float
    b: float
    c: float

    field(init=False)  # [invalid-field-call]

    def __post_init__(self):
        self.c = self.a + self.b


print(field(init=False))  # [invalid-field-call]



# Option 1: add explicit 'else'
def myfunc(shall_continue: bool, shall_exit: bool):
    if shall_continue:
        if input("Are you sure?") == "y":
            print("Moving on.")
        else:
            pass
    elif shall_exit:
        print("Exiting.")


# Option 2: extract function
def user_confirmation():
    if input("Are you sure?") == "y":
        print("Moving on.")


def myfunc2(shall_continue: bool, shall_exit: bool):
    if shall_continue:
        user_confirmation()
    elif shall_exit:
        print("Exiting.")



# pylint: disable=invalid-name, missing-docstring, redundant-u-string-prefix, line-too-long, superfluous-parens

# Basic test with a list
TEST_LIST1 = ['a' 'b']  # [implicit-str-concat]
# Testing with unicode strings in a tuple, with a comma AFTER concatenation
TEST_LIST2 = (u"a" u"b", u"c")  # [implicit-str-concat]
# Testing with raw strings in a set, with a comma BEFORE concatenation
TEST_LIST3 = {r'''a''', r'''b''' r'''c'''}  # [implicit-str-concat]
# Testing that only ONE warning is generated when string concatenation happens
# in the middle of a list
TEST_LIST4 = ["""a""", """b""" """c""", """d"""]  # [implicit-str-concat]

print("Lorem ipsum " "dolor sit amet")  # [implicit-str-concat]
print('a', 'bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb' 'ccc')  # [implicit-str-concat]

# The following shouldn't raise a warning because string literals are
# on different lines
TEST_LIST5 = ('a', 'b'
              'c')

# The following shouldn't raise a warning because of the escaped newline
TEST_LIST6 = ('bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb \
              ccc')

# But we should emit when there is an actual juxtaposition
# +1: [implicit-str-concat]
TEST_LIST7 = ('a' 'bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb \
              ccc')

# No warning for bytes
TEST_LIST8 = [b'A' b'B']

# There's an internal juxtaposition on the second line in a valid multiline
# TODO #6444: raise implicit-str-concat on the second line pylint: disable=fixme
print(
    "Lorem ipsum dolor sit amet, consectetur adipiscing elit,"
    " sed do eiusmod tempor " "incididunt ut labore et dolore "
    "magna aliqua. Ut enim ad minim veniam, quis nostrud "
    "exercitation ullamco laboris nisi ut aliquip ex ea "
)

with open("myfile.txt" "a+b", encoding="utf8") as f:  # [implicit-str-concat]
    content = f.read()



import random


def distribute_candies(children: list[Child], candies_per_child: int):
    total_candies = len(children) * candies_per_child
    eaten_candies = 0
    for child in children:
        eaten_candies += _distribute_candies_to_child(candies_per_child, child)
    return eaten_candies, total_candies


def _distribute_candies_to_child(candies_per_child: int, child: Child):
    # If a child eat more than 1 candies they're going to eat all
    # the candies for sure
    eaten_for_child = random.choices([0, 1, candies_per_child])
    print(f"Child {child} gets {candies_per_child} candies and eat {eaten_for_child}")
    remaining_candies_for_children = child.eat_candies(eaten_for_child)
    if remaining_candies_for_children == 0:
        print(f"All the candies have been devoured by {child.name}!")
    else:
        print(f"{child.name} still have {remaining_candies_for_children} candies left.")
    return eaten_for_child



# pylint: disable=missing-docstring,redefined-builtin, consider-using-f-string, unnecessary-direct-lambda-call, broad-exception-raised

from typing import NoReturn


def do_stuff(some_random_list):
    for var in some_random_list:
        pass
    return var # [undefined-loop-variable]


def do_else(some_random_list):
    for var in some_random_list:
        if var == 42:
            break
    else:
        var = 84
    return var

__revision__ = 'yo'

TEST_LC = [C for C in __revision__ if C.isalpha()]
B = [B for B in  __revision__ if B.isalpha()]
VAR2 = B # nor this one

for var1, var2 in TEST_LC:
    var1 = var2 + 4
VAR3 = var1 # [undefined-loop-variable]

for note in __revision__:
    note.something()
for line in __revision__:
    for note in line:
        A = note.anotherthing()


for x in []:
    pass
for x in range(3):
    VAR5 = (lambda: x)()


def do_stuff_with_a_list():
    for var in [1, 2, 3]:
        pass
    return var


def do_stuff_with_a_set():
    for var in {1, 2, 3}:  # pylint: disable=use-sequence-for-iteration
        pass
    return var


def do_stuff_with_a_dict():
    for var in {1: 2, 3: 4}:
        pass
    return var


def do_stuff_with_a_tuple():
    for var in (1, 2, 3):
        pass
    return var


def do_stuff_with_a_range():
    for var in range(1, 2):
        pass
    return var


def do_stuff_with_redefined_range():
    def range(key):
        yield from [1, key]
    for var in range(3):
        pass
    return var # [undefined-loop-variable]


def test(content):
    # https://github.com/pylint-dev/pylint/issues/3711
    def handle_line(layne):
        if "X" in layne:
            layne = layne.replace("X", "Y")
        elif "Y" in layne:  # line 5
            layne = '{}'.format(layne)
        elif "Z" in layne:  # line 7
            layne = f'{layne}'
        else:
            layne = '%s' % layne  # line 10

    for layne in content.split('\n'):
        handle_line(layne)


def for_else_returns(iterable):
    for thing in iterable:
        break
    else:
        return
    print(thing)


def for_else_raises(iterable):
    for thing in iterable:
        break
    else:
        raise Exception
    print(thing)


def for_else_break(iterable):
    while True:
        for thing in iterable:
            break
        else:
            break
        print(thing)


def for_else_continue(iterable):
    while True:
        for thing in iterable:
            break
        else:
            continue
        print(thing)


def for_else_no_return(iterable):
    def fail() -> NoReturn:
        ...

    while True:
        for thing in iterable:
            break
        else:
            fail()
        print(thing)


lst = []
lst2 = [1, 2, 3]

for item in lst:
    pass

bigger = [
    [
        x for x in lst2 if x > item
    ]
    for item in lst
]


def lambda_in_first_of_two_loops():
    """https://github.com/pylint-dev/pylint/issues/6419"""
    my_list = []
    for thing in my_list:
        print_it = lambda: print(thing)  # pylint: disable=cell-var-from-loop, unnecessary-lambda-assignment
        print_it()

    for thing in my_list:
        print(thing)


def variable_name_assigned_in_body_of_second_loop():
    for alias in tuple(bigger):
        continue
    for _ in range(3):
        alias = True
        if alias:
            print(alias)


def use_enumerate():
    """https://github.com/pylint-dev/pylint/issues/6593"""
    for i, num in enumerate(range(3)):
        pass
    print(i, num)


def use_enumerate_in_ternary_expression():
    """https://github.com/pylint-dev/pylint/issues/7131"""
    for i, num in enumerate(range(3)) if __revision__ else enumerate(range(4)):
        pass
    print(i, num)


def find_even_number(container):
    """https://github.com/pylint-dev/pylint/pull/6923#discussion_r895134495"""
    for something in container:
        if something % 2 == 0:
            break
    return something  # [undefined-loop-variable]



def integer_sum(a: int, b: int):  # [missing-return-type-doc]
    """Returns sum of two integers
    :param a: first integer
    :param b: second integer
    :return: sum of parameters a and b
    """
    return a + b



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from __future__ import annotations

import itertools
from pathlib import Path

import pytest

import pylint.checkers.unicode

SEARCH_DICT_BYTE_UTF8 = {
    char.unescaped.encode("utf-8"): char for char in pylint.checkers.unicode.BAD_CHARS
}


@pytest.mark.parametrize(
    "line, expected, search_dict",
    [
        # Test special carrier return cases
        pytest.param(
            "valid windows\r\n",
            {},
            pylint.checkers.unicode.BAD_ASCII_SEARCH_DICT,
            id="valid-windows",
        ),
        pytest.param(
            b"TOTO = ('Caf\xe9', 'Caf\xe9', 'Caf\xe9')\r\n",
            {},
            SEARCH_DICT_BYTE_UTF8,
            id="valid-windows-bytes",
        ),
        pytest.param(
            "invalid\r windows\r\n",
            {7: pylint.checkers.unicode.BAD_ASCII_SEARCH_DICT["\r"]},
            pylint.checkers.unicode.BAD_ASCII_SEARCH_DICT,
            id="invalid-carrier-return-windows",
        ),
        pytest.param(
            "invalid\r linux\n",
            {7: pylint.checkers.unicode.BAD_ASCII_SEARCH_DICT["\r"]},
            pylint.checkers.unicode.BAD_ASCII_SEARCH_DICT,
            id="invalid-carrier-return-linux",
        ),
        pytest.param(
            b"invalid\r windows\r\n",
            {7: pylint.checkers.unicode.BAD_ASCII_SEARCH_DICT["\r"]},
            SEARCH_DICT_BYTE_UTF8,
            id="invalid-carrier-return-windows-bytes",
        ),
        pytest.param(
            b"invalid\r linux\n",
            {7: pylint.checkers.unicode.BAD_ASCII_SEARCH_DICT["\r"]},
            SEARCH_DICT_BYTE_UTF8,
            id="invalid-carrier-return-linux-bytes",
        ),
        # Auto test Linux all remaining Linux cases ...
        *(
            pytest.param(
                f"invalid{char.unescaped} back\n",
                {7: char},
                pylint.checkers.unicode.BAD_ASCII_SEARCH_DICT,
                id=f"invalid-{char.name}-linux",
            )
            for char in pylint.checkers.unicode.BAD_CHARS
            if char.unescaped != "\r"
        ),
        # ... also byte encoded
        *(
            pytest.param(
                f"invalid{char.unescaped} back\n".encode("ASCII"),
                {7: char},
                SEARCH_DICT_BYTE_UTF8,
                id=f"invalid-{char.name}-linux-bytes",
            )
            for char in pylint.checkers.unicode.BAD_CHARS[:-1]
            if char.unescaped != "\r"
        ),
        # Test all remaining windows cases ...
        *(
            pytest.param(
                f"invalid{char.unescaped} back\r\n",
                {7: char},
                pylint.checkers.unicode.BAD_ASCII_SEARCH_DICT,
                id=f"invalid-{char.name}-windows",
            )
            for char in pylint.checkers.unicode.BAD_CHARS
            if char.unescaped != "\r"
        ),
        # ... also byte encoded
        *(
            pytest.param(
                f"invalid{char.unescaped} back\r\n".encode("ASCII"),
                {7: char},
                SEARCH_DICT_BYTE_UTF8,
                id=f"invalid-{char.name}-windows-bytes",
            )
            for char in pylint.checkers.unicode.BAD_CHARS[:-1]
            if char.unescaped != "\r"
        ),
    ],
)
def test_map_positions_to_result(
    line: pylint.checkers.unicode._StrLike,
    expected: dict[int, pylint.checkers.unicode._BadChar],
    search_dict: dict[
        pylint.checkers.unicode._StrLike, pylint.checkers.unicode._BadChar
    ],
) -> None:
    """Test all possible outcomes for map position function in UTF-8 and ASCII."""
    if isinstance(line, bytes):
        newline = b"\n"
    else:
        newline = "\n"
    assert (
        pylint.checkers.unicode._map_positions_to_result(
            line, search_dict, new_line=newline
        )
        == expected
    )


@pytest.mark.parametrize(
    "line",
    [
        pytest.param("1234567890", id="no_line_ending"),
        pytest.param(b"1234567890", id="no_line_ending_byte"),
        pytest.param("1234567890\n", id="linux"),
        pytest.param(b"1234567890\n", id="linux_byte"),
        pytest.param("1234567890\r\n", id="windows"),
        pytest.param(b"1234567890\r\n", id="windows_byte"),
        pytest.param("12345678\n\r", id="wrong_order"),
        pytest.param(b"12345678\n\r", id="wrong_order_byte"),
    ],
)
def test_line_length(line: pylint.checkers.unicode._StrLike) -> None:
    assert pylint.checkers.unicode._line_length(line, "utf-8") == 10


@pytest.mark.parametrize(
    "line",
    [
        pytest.param("1234567890", id="no_line_ending"),
        pytest.param("1234567890\n", id="linux"),
        pytest.param("1234567890\r\n", id="windows"),
        pytest.param("12345678\n\r", id="wrong_order"),
    ],
)
def test_line_length_utf16(line: str) -> None:
    assert pylint.checkers.unicode._line_length(line.encode("utf-16"), "utf-16") == 10


@pytest.mark.parametrize(
    "line",
    [
        pytest.param("1234567890", id="no_line_ending"),
        pytest.param("1234567890\n", id="linux"),
        pytest.param("1234567890\r\n", id="windows"),
        pytest.param("12345678\n\r", id="wrong_order"),
    ],
)
def test_line_length_utf32(line: str) -> None:
    assert pylint.checkers.unicode._line_length(line.encode("utf-32"), "utf-32") == 10


@pytest.mark.parametrize(
    "codec, expected",
    [
        ("utf-8sig", "utf-8"),
        ("utf8", "utf-8"),
        ("utf 8", "utf-8"),
        ("utf-8", "utf-8"),
        ("utf-8", "utf-8"),
        ("utf-16", "utf-16"),
        ("utf-32", "utf-32"),
        ("utf 16", "utf-16"),
        ("utf 32", "utf-32"),
        ("utf 16 LE", "utf-16le"),
        ("utf 32-BE", "utf-32be"),
        ("UTF-32", "utf-32"),
        ("UTF-32-le", "utf-32le"),
        ("UTF-16 LE", "utf-16le"),
        ("UTF-16BE", "utf-16be"),
        ("UTF8", "utf-8"),
        ("Latin1", "latin1"),
        ("ASCII", "ascii"),
    ],
)
def test__normalize_codec_name(codec: str, expected: str) -> None:
    assert pylint.checkers.unicode._normalize_codec_name(codec) == expected


@pytest.mark.parametrize(
    "codec, line_ending, final_new_line",
    [
        pytest.param(
            codec,
            line_ending[0],
            final_nl[0],
            id=f"{codec}_{line_ending[1]}_{final_nl[1]}",
        )
        for codec, line_ending, final_nl in itertools.product(
            (
                "utf-8",
                "utf-16",
                "utf-16le",
                "utf-16be",
                "utf-32",
                "utf-32le",
                "utf-32be",
            ),
            (("\n", "linux"), ("\r\n", "windows")),
            ((True, "final_nl"), (False, "no_final_nl")),
        )
    ],
)
def test___fix_utf16_32_line_stream(
    tmp_path: Path, codec: str, line_ending: str, final_new_line: bool
) -> None:
    """Content of stream should be the same as should be the length."""

    def decode_line(line: bytes, codec: str) -> str:
        return line.decode(codec)

    file = tmp_path / "test.txt"

    content = [
        f"line1{line_ending}",
        f"# Line 2{line_ending}",
        f"łöł{line_ending}",
        f"last line{line_ending if final_new_line else ''}",
    ]

    text = "".join(content)
    encoded = text.encode(codec)

    file.write_bytes(encoded)

    gathered = b""
    collected = []
    with file.open("rb") as f:
        for line in pylint.checkers.unicode._fix_utf16_32_line_stream(f, codec):
            gathered += line
            collected.append(decode_line(line, codec))

    # Test content equality
    assert collected == content
    # Test byte equality
    assert gathered == encoded


@pytest.mark.parametrize(
    "codec, expected",
    [
        ("utf-32", 4),
        ("utf-32-le", 4),
        ("utf-16", 2),
        ("utf-8", 1),
        ("latin1", 1),
        ("ascii", 1),
    ],
)
def test__byte_to_str_length(codec: str, expected: int) -> None:
    assert pylint.checkers.unicode._byte_to_str_length(codec) == expected



import requests


def fetch_data():
    for i in range(1, 6):
        print(f"Attempt {i}...")
        try:
            return requests.get("https://example.com/data")
        except requests.exceptions.RequestException:
            pass



"""Test for the regression on inference of self referential __len__
Reported in https://github.com/pylint-dev/pylint/issues/5244
"""
# pylint: disable=missing-class-docstring, missing-function-docstring


class MyClass:
    def some_func(self):
        return lambda: 42

    def __len__(self):
        return len(self.some_func())



from __future__ import annotations

from typing import TYPE_CHECKING

from astroid import nodes

from pylint.checkers import BaseRawFileChecker

if TYPE_CHECKING:
    from pylint.lint import PyLinter


class MyRawChecker(BaseRawFileChecker):
    r"""Check for line continuations with '\' instead of using triple
    quoted string or parenthesis.
    """

    name = "custom_raw"
    msgs = {
        "W9901": (
            "use \\ for line continuation",
            "backslash-line-continuation",
            (
                "Used when a \\ is used for a line continuation instead"
                " of using triple quoted string or parenthesis."
            ),
        )
    }
    options = ()

    def process_module(self, node: nodes.Module) -> None:
        """Process a module.

        the module's content is accessible via node.stream() function
        """
        with node.stream() as stream:
            for lineno, line in enumerate(stream):
                if line.rstrip().endswith("\\"):
                    self.add_message("backslash-line-continuation", line=lineno)


def register(linter: PyLinter) -> None:
    """This required method auto registers the checker during initialization.

    :param linter: The linter to register the checker to.
    """
    linter.register_checker(MyRawChecker(linter))



""" docstring for file clientmodule.py """
from data.suppliermodule_test import Interface, DoNothing, DoNothing2

class Ancestor:
    """ Ancestor method """
    cls_member = DoNothing()

    def __init__(self, value):
        local_variable = 0
        self.attr = 'this method shouldn\'t have a docstring'
        self.__value = value

    def get_value(self):
        """ nice docstring ;-) """
        return self.__value

    def set_value(self, value):
        self.__value = value
        return 'this method shouldn\'t have a docstring'

class Specialization(Ancestor):
    TYPE = 'final class'
    top = 'class'

    def __init__(self, value, _id, relation2: DoNothing2):
        Ancestor.__init__(self, value)
        self._id = _id
        self.relation = DoNothing()
        self.relation2 = relation2

    @classmethod
    def from_value(cls, value: int):
        return cls(value, 0, DoNothing2())

    @staticmethod
    def transform_value(value: int) -> int:
        return value * 2

    def increment_value(self) -> None:
        self.set_value(self.get_value() + 1)



def puppies(head: str, tail: str):
    """Print puppy's details.

    :param head: description of the head of the dog
    :param tail: description of the tail of the dog
    """
    print(head, tail)



"""Regression test for https://github.com/pylint-dev/pylint/issues/4692."""

# We can't use click like in the issue because the crash
# does not appear if click is installed (astroid can analyse it)
import notclick  # [import-error]


for name, item in notclick.__dict__.items():
    _ = isinstance(item, notclick.Command) and item != 'foo'



"""static method with non ascii characters"""


class OkayClass:
    """Class Docstring"""

    def public(self):
        """Say it load"""

    @staticmethod
    def umlaut_ä():  # [non-ascii-name]
        """Say ä"""
        return "ä"


# Usage should not raise a second error
OkayClass.umlaut_ä()



# pylint: disable=missing-docstring,too-few-public-methods

import foo.sub  # [import-error]


class Failed(metaclass=foo.sub.Metaclass):
    pass


class FailedTwo(metaclass=ab.ABCMeta):  # [undefined-variable]
    pass


class FailedThree(metaclass=foo.sob.Metaclass):
    pass


assert foo.sub.value is None



class ContextManager:
    def __enter__(self):
        pass

    def __exit__(self, *exc):
        pass


async def foo():
    async with ContextManager():  # [not-async-context-manager]
        pass



"""https://github.com/pylint-dev/pylint/issues/2615"""
def main():
    """When evaluating except blocks, assume try statements fail."""
    try:
        res = 1 / 0
        res = 42
        if main():
            res = None
        with open(__file__, encoding="utf-8") as opened_file:
            res = opened_file.readlines()
    except ZeroDivisionError:
        print(res)  # [used-before-assignment]
    print(res)


def nested_except_blocks():
    """Assignments in an except are tested against possibly failing
    assignments in try blocks at two different nesting levels."""
    try:
        res = 1 / 0
        res = 42
        if main():
            res = None
        with open(__file__, encoding="utf-8") as opened_file:
            res = opened_file.readlines()
    except ZeroDivisionError:
        try:
            more_bad_division = 1 / 0
        except ZeroDivisionError:
            print(more_bad_division)  # [used-before-assignment]
            print(res)  # [used-before-assignment]
    print(res)


def consecutive_except_blocks():
    """An assignment assumed to execute in one Try should continue to be
    assumed to execute in a consecutive Try.
    """
    try:
        res = 100
    except ZeroDivisionError:
        pass
    try:
        pass
    except ValueError:
        print(res)


def name_earlier_in_except_block():
    """Permit the name that might not have been assigned during the try block
    to be defined inside a conditional inside the except block.
    """
    try:
        res = 1 / 0
    except ZeroDivisionError:
        if main():
            res = 10
        else:
            res = 11
        print(res)



#!/usr/bin/env python

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Script used to generate the extensions file before building the actual documentation."""

from __future__ import annotations

import os
import re
import sys
from typing import Any, TypedDict

import sphinx
from sphinx.application import Sphinx

from pylint.checkers import BaseChecker
from pylint.constants import MAIN_CHECKER_NAME
from pylint.lint import PyLinter
from pylint.typing import MessageDefinitionTuple, OptionDict, ReportsCallable
from pylint.utils import get_rst_title


class _CheckerInfo(TypedDict):
    """Represents data about a checker."""

    checker: BaseChecker
    options: list[tuple[str, OptionDict, Any]]
    msgs: dict[str, MessageDefinitionTuple]
    reports: list[tuple[str, str, ReportsCallable]]
    doc: str
    module: str


# pylint: disable-next=unused-argument
def builder_inited(app: Sphinx | None) -> None:
    """Output full documentation in ReST format for all extension modules."""
    # PACKAGE/docs/exts/pylint_extensions.py --> PACKAGE/
    base_path = os.path.dirname(
        os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    )
    # PACKAGE/ --> PACKAGE/pylint/extensions
    ext_path = os.path.join(base_path, "pylint", "extensions")
    modules = []
    doc_files: dict[str, str] = {}
    for filename in os.listdir(ext_path):
        name, ext = os.path.splitext(filename)
        if name[0] == "_":
            continue
        if ext == ".py":
            modules.append(f"pylint.extensions.{name}")
        elif ext == ".rst":
            doc_files["pylint.extensions." + name] = os.path.join(ext_path, filename)
    modules.sort()
    if not modules:
        sys.exit("No Pylint extensions found?")

    linter = PyLinter()
    linter.load_plugin_modules(modules)

    extensions_doc = os.path.join(
        base_path, "doc", "user_guide", "checkers", "extensions.rst"
    )
    with open(extensions_doc, "w", encoding="utf-8") as stream:
        stream.write(get_rst_title("Optional checkers", "="))
        stream.write(
            """
.. This file is auto-generated. Make any changes to the associated
.. docs extension in 'doc/exts/pylint_extensions.py'.

"""
        )
        stream.write("Pylint provides the following optional plugins:\n\n")
        for module in modules:
            stream.write(f"- :ref:`{module}`\n")
        stream.write("\n")
        stream.write(
            "You can activate any or all of these extensions "
            "by adding a ``load-plugins`` line to the ``MAIN`` "
            "section of your ``.pylintrc``, for example::\n"
        )
        stream.write(
            "\n    load-plugins=pylint.extensions.docparams,"
            "pylint.extensions.docstyle\n\n"
        )

        # Print checker documentation to stream
        by_checker = get_plugins_info(linter, doc_files)
        max_len = len(by_checker)
        for i, checker_information in enumerate(sorted(by_checker.items())):
            checker, information = checker_information
            j = -1
            checker = information["checker"]
            if i == max_len - 1:
                # Remove the \n\n at the end of the file
                j = -3
            print(
                checker.get_full_documentation(
                    msgs=information["msgs"],
                    options=information["options"],
                    reports=information["reports"],
                    doc=information["doc"],
                    module=information["module"],
                    show_options=False,
                )[:j],
                file=stream,
            )


def get_plugins_info(
    linter: PyLinter, doc_files: dict[str, str]
) -> dict[BaseChecker, _CheckerInfo]:
    by_checker: dict[BaseChecker, _CheckerInfo] = {}
    for checker in linter.get_checkers():
        if checker.name == MAIN_CHECKER_NAME:
            continue
        module = checker.__module__
        # Plugins only - skip over core checkers
        if re.match("pylint.checkers", module):
            continue
        # Find any .rst documentation associated with this plugin
        doc = ""
        doc_file = doc_files.get(module)
        if doc_file:
            with open(doc_file, encoding="utf-8") as f:
                doc = f.read()
        try:
            by_checker[checker]["checker"] = checker
            by_checker[checker]["options"] += checker._options_and_values()
            by_checker[checker]["msgs"].update(checker.msgs)
            by_checker[checker]["reports"] += checker.reports
            by_checker[checker]["doc"] += doc
            by_checker[checker]["module"] += module
        except KeyError:
            by_checker[checker] = _CheckerInfo(
                checker=checker,
                options=list(checker._options_and_values()),
                msgs=dict(checker.msgs),
                reports=list(checker.reports),
                doc=doc,
                module=module,
            )
    return by_checker


def setup(app: Sphinx) -> dict[str, str | bool]:
    app.connect("builder-inited", builder_inited)
    return {"version": sphinx.__display_version__, "parallel_read_safe": True}


if __name__ == "__main__":
    builder_inited(None)



class Student:
    __slots__ = ("name",)

    def __init__(self, name, surname):
        self.name = name
        self.surname = surname  # [assigning-non-slot]
        self.setup()

    def setup(self):
        pass



"""Tests for useless suppressions"""
# pylint: enable=useless-suppression, line-too-long
# pylint: disable=unused-import, wrong-import-order, wrong-import-position

# False positive for wrong-import-order
# Reported in https://github.com/pylint-dev/pylint/issues/2366
from pylint import run_pylint
import astroid

# False-positive for 'line-too-long'
# Reported in https://github.com/pylint-dev/pylint/issues/4212
VAR = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" # pylint: disable=line-too-long

# False-positive for 'wrong-import-order'
# Reported in https://github.com/pylint-dev/pylint/issues/5219
import os



class Fruit:
    def brew(self, ingredient_name: str):
        print(f"Brewing a {type(self)} with {ingredient_name}")


class Apple(Fruit): ...


class Orange(Fruit):
    def brew(self, flavor: str):  # [arguments-renamed]
        print(f"Brewing an orange with {flavor}")


for fruit, ingredient_name in [[Orange(), "thyme"], [Apple(), "cinnamon"]]:
    fruit.brew(ingredient_name=ingredient_name)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from __future__ import annotations

import os
import re
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Any

import pytest
from pytest import CaptureFixture

from pylint.config.exceptions import ArgumentPreprocessingError
from pylint.interfaces import CONFIDENCE_LEVEL_NAMES
from pylint.lint import Run as LintRun
from pylint.testutils import create_files
from pylint.testutils._run import _Run as Run
from pylint.testutils.configuration_test import run_using_a_configuration_file

HERE = Path(__file__).parent.absolute()
REGRTEST_DATA_DIR = HERE / ".." / "regrtest_data"
EMPTY_MODULE = REGRTEST_DATA_DIR / "empty.py"
FIXME_MODULE = REGRTEST_DATA_DIR / "fixme.py"


def check_configuration_file_reader(
    runner: LintRun,
    expected_disabled: set[str] | None = None,
    expected_jobs: int = 10,
    expected_reports_truthey: bool = True,
) -> None:
    """Check that what we initialized the linter with what was expected."""
    if expected_disabled is None:
        # "logging-not-lazy" and "logging-format-interpolation"
        expected_disabled = {"W1201", "W1202"}
    for msgid in expected_disabled:
        assert not runner.linter.is_message_enabled(msgid)
    assert runner.linter.config.jobs == expected_jobs
    assert bool(runner.linter.config.reports) == expected_reports_truthey


def test_can_read_toml_env_variable(tmp_path: Path, file_to_lint_path: str) -> None:
    """We can read and open a properly formatted toml file."""
    config_file = tmp_path / "pyproject.toml"
    config_file.write_text(
        """
[tool.pylint."messages control"]
disable = "logging-not-lazy,logging-format-interpolation"
jobs = "10"
reports = "yes"
"""
    )
    env_var = "tmp_path_env"
    os.environ[env_var] = str(config_file)
    mock_exit, _, runner = run_using_a_configuration_file(
        f"${env_var}", file_to_lint_path
    )
    mock_exit.assert_called_once_with(0)
    check_configuration_file_reader(runner)


def test_unknown_message_id(capsys: CaptureFixture) -> None:
    """Check that we correctly raise a message on an unknown id."""
    Run([str(EMPTY_MODULE), "--disable=12345"], exit=False)
    output = capsys.readouterr()
    assert "Command line:1:0: W0012: Unknown option value for '--disable'" in output.out


def test_unknown_option_name(capsys: CaptureFixture) -> None:
    """Check that we correctly raise a message on an unknown option."""
    with pytest.raises(SystemExit):
        Run([str(EMPTY_MODULE), "--unknown-option=yes"], exit=False)
    output = capsys.readouterr()
    assert "usage: pylint" in output.err
    assert "Unrecognized option" in output.err


def test_unknown_short_option_name(capsys: CaptureFixture) -> None:
    """Check that we correctly raise a message on an unknown short option."""
    with pytest.raises(SystemExit):
        Run([str(EMPTY_MODULE), "-Q"], exit=False)
    output = capsys.readouterr()
    assert "usage: pylint" in output.err
    assert "Unrecognized option" in output.err


def test_unknown_confidence(capsys: CaptureFixture) -> None:
    """Check that we correctly error an unknown confidence value."""
    with pytest.raises(SystemExit):
        Run([str(EMPTY_MODULE), "--confidence=UNKNOWN_CONFIG"], exit=False)
    output = capsys.readouterr()
    assert "argument --confidence: UNKNOWN_CONFIG should be in" in output.err


def test_empty_confidence() -> None:
    """An empty confidence value indicates all errors should be emitted."""
    r = Run([str(EMPTY_MODULE), "--confidence="], exit=False)
    assert r.linter.config.confidence == CONFIDENCE_LEVEL_NAMES


def test_unknown_yes_no(capsys: CaptureFixture) -> None:
    """Check that we correctly error on an unknown yes/no value."""
    with pytest.raises(SystemExit):
        Run([str(EMPTY_MODULE), "--reports=maybe"], exit=False)
    output = capsys.readouterr()
    assert "Invalid yn value 'maybe', should be in " in output.err


def test_unknown_py_version(capsys: CaptureFixture) -> None:
    """Check that we correctly error on an unknown python-version."""
    with pytest.raises(SystemExit):
        Run([str(EMPTY_MODULE), "--py-version=the-newest"], exit=False)
    output = capsys.readouterr()
    assert "the-newest has an invalid format, should be a version string." in output.err


CSV_REGEX_COMMA_CASES = [
    ("foo", ["foo"]),
    ("foo,bar", ["foo", "bar"]),
    ("foo, bar", ["foo", "bar"]),
    ("foo, bar{1,3}", ["foo", "bar{1,3}"]),
]


@pytest.mark.parametrize("in_string,expected", CSV_REGEX_COMMA_CASES)
def test_csv_regex_comma_in_quantifier(in_string: str, expected: list[str]) -> None:
    """Check that we correctly parse a comma-separated regex when there are one
    or more commas within quantifier expressions.
    """

    def _template_run(in_string: str) -> list[re.Pattern[Any]]:
        r = Run(
            [str(EMPTY_MODULE), rf"--bad-names-rgx={in_string}"],
            exit=False,
        )
        bad_names_rgxs: list[re.Pattern[Any]] = r.linter.config.bad_names_rgxs
        return bad_names_rgxs

    assert _template_run(in_string) == [re.compile(regex) for regex in expected]


def test_regex_error(capsys: CaptureFixture) -> None:
    """Check that we correctly error when an an option is passed whose value is an invalid regular expression."""
    with pytest.raises(SystemExit):
        Run(
            [str(EMPTY_MODULE), r"--function-rgx=[\p{Han}a-z_][\p{Han}a-z0-9_]{2,30}$"],
            exit=False,
        )
    output = capsys.readouterr()

    assertString = (
        r"Error in provided regular expression: [\p{Han}a-z_][\p{Han}a-z0-9_]{2,30}$ "
        r"beginning at index 1: bad escape \p"
    )
    assert assertString in output.err


def test_csv_regex_error(capsys: CaptureFixture) -> None:
    """Check that we correctly error when an option is passed and one
    of its comma-separated regular expressions values is an invalid regular expression.
    """
    with pytest.raises(SystemExit):
        Run(
            [str(EMPTY_MODULE), r"--bad-names-rgx=(foo{1,}, foo{1,3}})"],
            exit=False,
        )
    output = capsys.readouterr()
    assert (
        r"Error in provided regular expression: (foo{1,} beginning at index 0: missing ), unterminated subpattern"
        in output.err
    )


def test_short_verbose(capsys: CaptureFixture) -> None:
    """Check that we correctly handle the -v flag."""
    Run([str(EMPTY_MODULE), "-v"], exit=False)
    output = capsys.readouterr()
    assert "Using config file" in output.err


def test_argument_separator() -> None:
    """Check that we support using '--' to separate argument types.

    Reported in https://github.com/pylint-dev/pylint/issues/7003.
    """
    runner = Run(["--", str(EMPTY_MODULE)], exit=False)
    assert not runner.linter.stats.by_msg


def test_clear_cache_post_run() -> None:
    modname = "changing.py"
    with TemporaryDirectory() as tmp_dir:
        create_files([modname], tmp_dir)
        module = tmp_dir + os.sep + modname
        # Run class does not produce the wanted failure
        # must use LintRun to get pylint.lint.Run
        run_before_edit = LintRun([module, "--clear-cache-post-run=y"], exit=False)
        with open(module, mode="a", encoding="utf-8") as f:
            f.write("undefined\n")
        run_after_edit = LintRun([module, "--clear-cache-post-run=y"], exit=False)

    assert not run_before_edit.linter.stats.by_msg
    assert run_after_edit.linter.stats.by_msg


def test_enable_all_disable_all_mutually_exclusive() -> None:
    with pytest.raises(ArgumentPreprocessingError):
        runner = Run(["--enable=all", "--disable=all", str(EMPTY_MODULE)], exit=False)

    runner = Run(["--enable=all", "--enable=all", str(EMPTY_MODULE)], exit=False)
    assert not runner.linter.stats.by_msg

    with pytest.raises(ArgumentPreprocessingError):
        run_using_a_configuration_file(
            HERE
            / "functional"
            / "toml"
            / "toml_with_mutually_exclusive_disable_enable_all.toml",
        )


def test_disable_before_enable_all_takes_effect() -> None:
    runner = Run(["--disable=fixme", "--enable=all", str(FIXME_MODULE)], exit=False)
    assert not runner.linter.stats.by_msg

    _, _, toml_runner = run_using_a_configuration_file(
        HERE
        / "functional"
        / "toml"
        / "toml_with_specific_disable_before_enable_all.toml",
    )
    assert not toml_runner.linter.is_message_enabled("fixme")


def test_enable_before_disable_all_takes_effect() -> None:
    runner = Run(["--enable=fixme", "--disable=all", str(FIXME_MODULE)], exit=False)
    assert runner.linter.stats.by_msg

    _, _, toml_runner = run_using_a_configuration_file(
        HERE
        / "functional"
        / "toml"
        / "toml_with_specific_enable_before_disable_all.toml",
    )
    assert toml_runner.linter.is_message_enabled("fixme")



# pylint: disable=too-few-public-methods, unnecessary-pass, unnecessary-dunder-call
"""Backend Base Classes for the schwelm user DB"""


class Aaa:
    """docstring"""
    def __init__(self):
        self.__setattr__('a', 'b')


    def one_public(self):
        """docstring"""
        pass

    def another_public(self):
        """docstring"""
        pass

class Bbb(Aaa):
    """docstring"""
    pass

class Ccc(Aaa):
    """docstring"""

    class Ddd(Aaa):
        """docstring"""
        pass

    class Eee(Ddd):
        """docstring"""
        pass



"""Test that classes inheriting directly from Protocol should not warn about abstract-method."""

# pylint: disable=too-few-public-methods,disallowed-name,invalid-name

from abc import abstractmethod, ABCMeta
from typing import Protocol, Literal


class FooProtocol(Protocol):
    """Foo Protocol"""

    @abstractmethod
    def foo(self) -> Literal["foo"]:
        """foo method"""

    def foo_no_abstract(self) -> Literal["foo"]:
        """foo not abstract method"""


class BarProtocol(Protocol):
    """Bar Protocol"""
    @abstractmethod
    def bar(self) -> Literal["bar"]:
        """bar method"""


class FooBarProtocol(FooProtocol, BarProtocol, Protocol):
    """FooBar Protocol"""

class BarParent(BarProtocol): # [abstract-method]
    """Doesn't subclass typing.Protocol directly"""

class IndirectProtocol(FooProtocol):  # [abstract-method]
    """Doesn't subclass typing.Protocol directly"""

class AbcProtocol(FooProtocol, metaclass=ABCMeta):
    """Doesn't subclass typing.Protocol but uses metaclass directly"""

class FooBar(FooBarProtocol):
    """FooBar object"""

    def bar(self) -> Literal["bar"]:
        return "bar"

    def foo(self) -> Literal["foo"]:
        return "foo"



# pylint: disable=wildcard-import,unused-import,invalid-name,import-error
# pylint: disable=bare-except,broad-except,wrong-import-order,ungrouped-imports,wrong-import-position
"""check nonexistent names imported are reported"""

import collections.tutu  # [no-name-in-module]
from collections import toto  # [no-name-in-module]
toto.yo()

from xml.etree import ElementTree
ElementTree.nonexistant_function()  # [no-member]
ElementTree.another.nonexistant.function()  # [no-member]


import sys
print(sys.stdout, 'hello world')
print(sys.stdoout, 'bye bye world')  # [no-member]


import re
re.finditer('*', 'yo')

from rie import *
from re import findiiter, compiile  # [no-name-in-module,no-name-in-module]

import os
'SOMEVAR' in os.environ  # [pointless-statement]

try:
    from collections import something
except ImportError:
    something = None

try:
    from collections import anything # [no-name-in-module]
except ValueError:
    anything = None

try:
    import collections.missing
except ImportError:
    pass

try:
    import collections.missing
except ModuleNotFoundError:
    pass

try:
    import collections.indeed_missing # [no-name-in-module]
except ValueError:
    pass

try:
    import collections.emit # [no-name-in-module]
except Exception:
    pass

try:
    import collections.emit1
except ImportError:
    pass

try:
    import collections.emit1
except ModuleNotFoundError:
    pass


try:
    if something:
        import collections.emit2 # [no-name-in-module]
except Exception:
    pass

from .no_self_argument import NoSelfArgument
from .no_self_argument import lala  # [no-name-in-module]
from .no_self_argument.bla import lala1 # [no-name-in-module]

# Check ignored-modules setting
from argparse import THIS_does_not_EXIST


# This captures the original failure in https://github.com/pylint-dev/pylint/issues/6497
# only if numpy is installed. We are not installing numpy on CI (for now)
from numpy.distutils.misc_util import is_sequence
from pydantic import BaseModel



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from pathlib import Path

HERE = Path(__file__).parent
REGR_DATA = HERE / ".." / "regrtest_data"



"""
'collections.abc.Callable' is broken inside Optional and Union types for Python 3.9.0
https://bugs.python.org/issue42965

Use 'typing.Callable' instead.

Don't emit 'deprecated-typing-alias' for 'Callable' if at least one replacement
would create broken instances.
"""
# pylint: disable=missing-docstring,unsubscriptable-object,invalid-name
from typing import Callable, Optional, Union

Alias1 = Optional[Callable[[int], None]]
Alias2 = Union[Callable[[int], None], None]

Alias3 = Optional[Callable[..., None]]
Alias4 = Union[Callable[..., None], None]
Alias5 = list[Callable[[int], None]]
Alias6 = Callable[[int], None]


def func1() -> Optional[Callable[[int], None]]:
    ...

def func2() -> Optional["Callable[[int], None]"]:
    ...



class Point:
    def __init__(self, x, y):
        """Represents a point in the xy-coordinate plane.

        :param x: x coordinate
        :param y: y coordinate
        """
        self.x = x
        self.y = y



# pylint: disable=missing-docstring, redundant-keyword-arg, invalid-name, line-too-long
import threading


threading.Thread(lambda: None).run()  # [bad-thread-instantiation]
threading.Thread(None, lambda: None)
threading.Thread(lambda: None, group=None)  # [bad-thread-instantiation]
threading.Thread()  # [bad-thread-instantiation]

threading.Thread(group=None, target=lambda: None).run()
threading.Thread(group=None, target=None, name=None, args=(), kwargs={})
threading.Thread(None, None, "name")

def thread_target(n):
    print(n ** 2)


thread = threading.Thread(thread_target, args=(10,))  # [bad-thread-instantiation]


kw = {'target_typo': lambda x: x}
threading.Thread(None, **kw)  # [unexpected-keyword-arg, bad-thread-instantiation]

threading.Thread(None, target_typo=lambda x: x)  # [unexpected-keyword-arg, bad-thread-instantiation]



# pylint: disable=missing-docstring,too-few-public-methods,useless-return,lost-exception,inconsistent-return-statements

def second_favorite():
    fruits = ["apple"]
    try:
        return fruits[1]
    finally:
        return fruits[0]  # [return-in-finally]


def all_favorites():
    fruits = ["orange", "kiwi", "pineapple"]

    for fruit in fruits:
        try:
            return fruit
        finally:
            return  # [return-in-finally]


def more_favorites():
    fruits = ["orange", "kiwi", "pineapple"]

    for fruit in fruits:
        try:
            return fruit
        finally:
            if len(fruit) > 7:
                return "too many fruits"  # [return-in-finally]


def even_more_favorites():
    fruits = ["orange", "kiwi", "pineapple"]

    for fruit in fruits:
        try:
            return fruit
        finally:
            if len(fruit) > 7:
                for fruit_name in fruits:
                    return f"please remove {fruit_name}"  # [return-in-finally]



"""Tests for super-init-not-called."""
# pylint: disable=too-few-public-methods, missing-class-docstring

import abc
import ctypes


class Foo(ctypes.BigEndianStructure):
    """This class should not emit a super-init-not-called warning.

    It previously did, because ``next(node.infer())`` was used in that checker's logic
    and the first inferred node was an Uninferable object, leading to this false positive.
    """

    def __init__(self):
        ctypes.BigEndianStructure.__init__(self)


class UninferableChild(UninferableParent):  # [undefined-variable]
    """An implementation that test if we don't crash on uninferable parents."""

    def __init__(self):
        ...


# Tests for not calling the init of a parent that does not define one
# but inherits it.
class GrandParentWithInit:
    def __init__(self):
        print(self)


class ParentWithoutInit(GrandParentWithInit):
    pass


class ChildOne(ParentWithoutInit, GrandParentWithInit):
    """Since ParentWithoutInit calls GrandParentWithInit it doesn't need to be called."""

    def __init__(self):
        GrandParentWithInit.__init__(self)


class ChildTwo(ParentWithoutInit):
    def __init__(self):
        ParentWithoutInit.__init__(self)


class ChildThree(ParentWithoutInit):
    def __init__(self):  # [super-init-not-called]
        ...


# Regression test as reported in
# https://github.com/pylint-dev/pylint/issues/6027
class MyUnion(ctypes.Union):
    def __init__(self):
        pass


# Should not be called on abstract __init__ methods
# https://github.com/pylint-dev/pylint/issues/3975
class Base:
    def __init__(self, param: int, param_two: str) -> None:
        raise NotImplementedError()


class Derived(Base):
    def __init__(self, param: int, param_two: str) -> None:
        self.param = param + 1
        self.param_two = param_two[::-1]


class AbstractBase(abc.ABC):
    def __init__(self, param: int) -> None:
        self.param = param + 1

    def abstract_method(self) -> str:
        """This needs to be implemented."""
        raise NotImplementedError()


class DerivedFromAbstract(AbstractBase):
    def __init__(self, param: int) -> None:  # [super-init-not-called]
        print("Called")

    def abstract_method(self) -> str:
        return "Implemented"


class DerivedFrom(UnknownParent):  # [undefined-variable]
    def __init__(self) -> None:
        print("Called")


class DerivedFromUnknownGrandparent(DerivedFrom):
    def __init__(self) -> None:
        DerivedFrom.__init__(self)



from importlib import import_module

from pylint.checkers import BaseChecker
from pylint.lint.pylinter import PyLinter


class ImportingChecker(BaseChecker):
    options = (
        (
            "settings-module",
            {
                "default": "settings",
                "type": "string",
                "metavar": "<settings module>"
            },
        ),
    )

    msgs = {
        "E9999": (
            "Importing checker error message",
            "importing-checker-error",
            "Importing checker error message",
        ),
    }

    def open(self) -> None:
        import_module(self.linter.config.settings_module)


def register(linter: "PyLinter") -> None:
    linter.register_checker(ImportingChecker(linter))



"""Test returns from structural pattern matching cases."""

def unpack(num) -> tuple[int, int]:
    """Return a tuple of integers."""
    match num:
        case 1:
            return 1, 1
        case _:
            return 0, 0

x, y = unpack(1)



"""Miscellaneous used-before-assignment cases"""
# pylint: disable=consider-using-f-string, missing-function-docstring
import datetime
import sys

MSG = "hello %s" % MSG  # [used-before-assignment]

MSG2 = "hello %s" % MSG2  # [used-before-assignment]

def outer():
    inner()  # [used-before-assignment]
    def inner():
        pass

outer()


class ClassWithProperty:  # pylint: disable=too-few-public-methods
    """This test depends on earlier and later defined module-level functions."""
    prop = property(redefine_time_import)  # [used-before-assignment]
    prop_defined_earlier = property(outer)


calculate(1.01, 2)  # [used-before-assignment]
def calculate(value1: int, value2: float) -> int:
    return value1 + value2


# pylint: disable=unused-import, wrong-import-position, import-outside-toplevel, reimported, redefined-outer-name, global-statement
import time
def redefine_time_import():
    print(time.time())  # [used-before-assignment]
    import time


def redefine_time_import_with_global():
    global time  # pylint: disable=invalid-name
    print(time.time())
    import time


# Control flow cases
FALSE = False
if FALSE:
    VAR2 = True
if VAR2:  # [used-before-assignment]
    pass

if FALSE:  # pylint: disable=simplifiable-if-statement
    VAR3 = True
elif VAR2:
    VAR3 = True
else:
    VAR3 = False
if VAR3:
    pass

if FALSE:
    VAR4 = True
elif VAR2:
    pass
else:
    VAR4 = False
if VAR4:  # [possibly-used-before-assignment]
    pass

if FALSE:
    VAR5 = True
elif VAR2:
    if FALSE:  # pylint: disable=simplifiable-if-statement
        VAR5 = True
    else:
        VAR5 = True
if VAR5:  # [possibly-used-before-assignment]
    pass

if FALSE:
    VAR6 = False
if VAR6:  # [used-before-assignment]
    pass


# Nested try
if FALSE:
    try:
        VAR7 = True
    except ValueError:
        pass
else:
    VAR7 = False
if VAR7:
    pass

if FALSE:
    try:
        VAR8 = True
    except ValueError as ve:
        print(ve)
        raise
else:
    VAR8 = False
if VAR8:
    pass

if FALSE:
    for i in range(5):
        VAR9 = i
        break
print(VAR9)

if FALSE:
    with open(__name__, encoding='utf-8') as f:
        VAR10 = __name__
print(VAR10)  # [used-before-assignment]

for num in [0, 1]:
    VAR11 = num
    if VAR11:
        VAR12 = False
print(VAR12)  # [possibly-used-before-assignment]

if input("This tests terminating functions: "):
    sys.exit()
else:
    VAR13 = 1
print(VAR13)

def turn_on2(**kwargs):
    """https://github.com/pylint-dev/pylint/issues/7873"""
    if "brightness" in kwargs:
        brightness = kwargs["brightness"]
        var, *args = (1, "set_dimmer_state", brightness)
    else:
        var, *args = (1, "restore_dimmer_state")

    print(var, *args)


# Variables guarded by the same test when used.

# Always false
if 1 in []:
    PERCENT = 20
    SALE = True

if 1 in []:
    print(PERCENT)

# Different test
if 1 in [1]:
    print(SALE)  # [used-before-assignment]


# Ambiguous, but same test
if not datetime.date.today():
    WAS_TODAY = True

if not datetime.date.today():
    print(WAS_TODAY)


# Different tests but same inferred values
# Need falsy values here
def give_me_zero():
    return 0

def give_me_nothing():
    return 0

if give_me_zero():
    WE_HAVE_ZERO = True
    ALL_DONE = True

if give_me_nothing():
    print(WE_HAVE_ZERO)


# Different tests, different values
def give_me_none():
    return None

if give_me_none():
    print(ALL_DONE)  # [used-before-assignment]


attr = 'test'  # pylint: disable=invalid-name
class T:  # pylint: disable=invalid-name, too-few-public-methods, undefined-variable
    '''Issue #8754, no crash from unexpected assignment between attribute and variable'''
    T.attr = attr


if outer():
    NOT_ALWAYS_DEFINED = True
print(NOT_ALWAYS_DEFINED)  # [used-before-assignment]


def inner_if_continues_outer_if_has_no_other_statements():
    for i in range(5):
        if isinstance(i, int):
            # Testing no assignment here, before the inner if
            if i % 2 == 0:
                order = None
            else:
                continue
        else:
            order = None
        print(order)



# pylint: disable=too-few-public-methods,missing-docstring,invalid-name
"""test detection of method which could be a function"""
from abc import ABC, abstractmethod
from typing import Protocol, overload


class Toto:
    """bla bal abl"""

    def __init__(self):
        self.aaa = 2

    def regular_method(self):
        """this method is a real method since it access to self"""
        self.function_method()

    def function_method(self):  # [no-self-use]
        """this method isn' a real method since it doesn't need self"""
        print('hello')

    async def async_regular_method(self):
        """this async method is a real method since it accesses self"""
        await self.async_function_method()

    async def async_function_method(self):  # [no-self-use]
        """this async method isn't a real method since it doesn't need self"""
        print('hello')

class Base:
    """an abstract class"""

    def __init__(self):
        self.aaa = 2

    def check(self, arg):
        """an abstract method, could not be a function"""
        raise NotImplementedError


class Sub(Base):
    """a concrete class"""

    def check(self, arg):
        """a concrete method, could not be a function since it need
        polymorphism benefits
        """
        return arg == 0

class Super:
    """same as before without abstract"""
    attr = 1
    def method(self):
        """regular"""
        print(self.attr)

class Sub1(Super):
    """override method with need for self"""
    def method(self):
        """no i can not be a function"""
        print(42)

    def __len__(self):
        """no i can not be a function"""
        return 42

    def __cmp__(self, other):
        """no i can not be a function"""
        print(42)

    def __copy__(self):
        return 24

    def __getstate__(self):
        return 42


class Prop:

    @property
    def count(self):
        """Don't emit no-self-use for properties.

        They can't be functions and they can be part of an
        API specification.
        """
        return 42


class A:
    def __init__(self):
        self.store = {}

    def get(self, key, default=None):
        return self.store.get(key, default)

class B(A):
    def get_memo(self, obj):
        return super().get(obj)


class C:
    def a(self, /):  # [no-self-use]
        ...

    # Disable with old error code
    # pylint: disable=use-symbolic-message-instead
    def b(self, /):  # pylint: disable=R0201
        ...


def func_a(self):  # pylint: disable=unused-argument
    pass


class Foo1(ABC):
    """Don't emit no-self-use for abstract methods."""

    @abstractmethod
    def a(self):
        pass

    def b(self):
        raise NotImplementedError

    def c(self):
        pass  # pass counts as abstract


class Foo2(Protocol):
    """Don't emit no-self-use for methods in Protocol classes."""

    def a(self):
        ...

class Foo3:
    """Don't emit no-self-use for overload methods."""

    @overload
    def a(self, var): ...

    @overload
    def a(self, var): ...

    def a(self, var):
        pass


class Foo4:
    """Other false positive cases."""

    @staticmethod
    def a(self):  # pylint: disable=unused-argument,bad-staticmethod-argument
        ...

    @staticmethod
    def b():
        ...

    @classmethod
    def c(self):  # pylint: disable=bad-classmethod-argument
        ...

    def d():  # pylint: disable=no-method-argument
        ...



# pylint: disable=missing-docstring

import socket


class SomeException(Exception):
    pass


class SubclassException(SomeException):
    pass


AliasException = SomeException

try:
    pass
except (SomeException, SomeException):  # [overlapping-except]
    pass

try:
    pass
except (SomeException, SubclassException):  # [overlapping-except]
    pass

try:
    pass
except (SomeException, AliasException):  # [overlapping-except]
    pass

try:
    pass
except (AliasException, SubclassException):  # [overlapping-except]
    pass

try:
    pass
# +1:[overlapping-except, overlapping-except, overlapping-except]
except (SomeException, AliasException, SubclassException):
    pass

try:
    pass
except (ArithmeticError, FloatingPointError):  # [overlapping-except]
    pass

try:
    pass
except (ValueError, UnicodeDecodeError):  # [overlapping-except]
    pass


try:
    pass
except (IOError, OSError):  # [overlapping-except]
    pass

try:
    pass
except (socket.error, OSError):  # [overlapping-except]
    pass

try:
    pass
except (ConnectionError, socket.error):  # [overlapping-except]
    pass



class Student:
    __slots__ = ("name", "surname")

    def __init__(self, name, surname):
        self.name = name
        self.surname = surname
        self.setup()

    def setup(self):
        pass



# pylint: disable=missing-function-docstring, missing-module-docstring

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from datetime import datetime


def func_two():
    second = datetime.now()  # [used-before-assignment]
    return second


def func():
    first: datetime
    first = datetime.now()  # [used-before-assignment]
    second = datetime.now()
    return first, second



from dataclasses import dataclass


@dataclass
class ThreeDChessPiece:
    x: int
    y: int
    z: int
    type: str


def three_d_chess_move(
    white: ThreeDChessPiece,
    black: ThreeDChessPiece,
    blue: ThreeDChessPiece,
    current_player,
):
    pass



# pylint: disable=missing-docstring, too-few-public-methods, invalid-name

# Inheritance diagram:
#       F
#      /
#  D  E
#   \/
#    B  C
#     \/
#      A
#
# Once `E` is pruned from the tree, we have:
#  D
#   \
#    B  C
#     \/
#      A
#
# By setting `max-parents=2`, we're able to check that tree-pruning works
# correctly; in the new diagram, `B` has only 1 parent, so it doesn't raise a
# message, and `A` has 3, so it does raise a message with the specific number
# of parents.

class F:
    """0 parents"""

class E(F):
    """1 parent"""

class D:
    """0 parents"""

class B(D, E):
    """3 parents"""

class C:
    """0 parents"""

class A(B, C): # [too-many-ancestors]
    """5 parents"""
